﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class AnimatedMesh : Entity
    {
        public List<int> sequences = new List<int>();
        /// <summary>
        /// Extracts part of specified sequence to a new one and returns its index. 
        /// B3D format supports only one sequence per file. You can arrange all animations into one in editor and then extact them in Xors3D 
        /// </summary>
        /// <param name="firstFrame">First frame of anim sequence to extract </param>
        /// <param name="lastFrame">Last frame of anim sequence to extract </param>
        /// <param name="seq">Animation sequence to extract from </param>
        /// <returns></returns>
        public int ExtractAnimSeq(int firstFrame, int lastFrame, int seq)
        {
            return Xors3d.ExtractAnimSeq(_handle, firstFrame, lastFrame, seq);
        }

        /// <summary>
        /// Appends an animation sequence from a file to an entity. Returns the animation sequence number added
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public int LoadAnimSeq(string path)
        {
            return Xors3d.LoadAnimSeq(_handle, path);
        }

        /// <summary>
        /// Sets entity's animation speed
        /// </summary>
        /// <param name="speed"></param>
        /// <param name="rootBone">Root bone in hierarhy, from that animation will be applied. In Blitz3D you may find bone by FindChild() command and animate it. In Xors3D you must simply pass its name in xAnimate() command. For hierarhy a new animation track will be created, so then you will need to get its info (like time, speed, etc) you must also pass a name of its root bone for correct results</param>
        public void SetAnimSpeed(float speed, string rootBone = "")
        {
            Xors3d.SetAnimSpeed(_handle, speed, rootBone);
        }

        /// <summary>
        /// Returns the specified entity's current animation speed. 
        /// This function must be used only with real skinned mesh, or it will return wrong values 
        /// </summary>
        /// <param name="rootBone"></param>
        public void AnimSpeed(string rootBone)
        {
            Xors3d.AnimSpeed(_handle, rootBone);
        }

        /// <summary>
        /// Returns true if the specified entity is currently animating. 
        /// This function must be used only with real skinned mesh, or it will return wrong values 
        /// </summary>
        /// <param name="rootBone">Root bone in hierarhy, from that animation will be applied. In Blitz3D you may find bone by FindChild() command and animate it. In Xors3D you must simply pass its name in xAnimate() command. For hierarhy a new animation track will be created, so then you will need to get its info (like time, speed, etc) you must also pass a name of its root bone for correct results</param>
        /// <returns></returns>
        public bool Animating(string rootBone = "")
        {
            return Xors3d.Animating(_handle, rootBone) == 1;
        }

        /// <summary>
        /// Returns the current animation time of an entity. 
        /// This function must be used only with real skinned mesh, or it will return wrong values 
        /// </summary>
        /// <param name="rootBone">Root bone in hierarhy, from that animation will be applied. In Blitz3D you may find bone by FindChild() command and animate it. In Xors3D you must simply pass its name in xAnimate() command. For hierarhy a new animation track will be created, so then you will need to get its info (like time, speed, etc) you must also pass a name of its root bone for correct results</param>
        /// <returns></returns>
        public float AnimTime(string rootBone)
        {
            return Xors3d.AnimTime(_handle, rootBone);
        }

        /// <summary>
        /// Animates an entity
        /// </summary>
        /// <param name="mode">Animating mode. See 'Animation playback types' for more information </param>
        /// <param name="speed">Animation speed. A negative speed will play the animation backwards </param>
        /// <param name="sequence">Initially, an entity loaded with xLoadAnimMesh() will have a single animation sequence. More sequences can be added using either xLoadAnimSeq() or xAddAnimSeq(). Animation sequences are numbered 0, 1, 2, etc </param>
        /// <param name="translate">A value of 0 will cause an instant 'leap' to the first frame, while values greater than 0 will cause a smooth transition </param>
        /// <param name="rootBone">Root bone in hierarhy, from that animation will be applied. In Blitz3D you may find bone by FindChild() command and animate it. In Xors3D you must simply pass its name in xAnimate() command. For hierarhy a new animation track will be created, so then you will need to get its info (like time, speed, etc) you must also pass a name of its root bone for correct results</param>
        public void Animate(AnimTypes mode = AnimTypes.LOOP, float speed = 1.0f, int sequence = 0, float translate = 0.0f, string rootBone = "")
        {
            Xors3d.Animate(_handle, (int)mode, speed, sequence, translate, rootBone);
        }

        /// <summary>
        /// Returns the specified entity's current animation sequence. 
        /// This function must be used only with real skinned mesh, or it will return wrong values 
        /// </summary>
        /// <param name="rootBone">Root bone in hierarhy, from that animation will be applied. In Blitz3D you may find bone by FindChild() command and animate it. In Xors3D you must simply pass its name in xAnimate() command. For hierarhy a new animation track will be created, so then you will need to get its info (like time, speed, etc) you must also pass a name of its root bone for correct results</param>
        /// <returns></returns>
        public int GetCurrentAnimSeq(string rootBone="")
        {
            return Xors3d.AnimSeq(_handle, rootBone);
        }

        /// <summary>
        /// Returns the length of the specified entity's current animation sequence. This function must be used only with real skinned mesh, or it will return wrong values 
        /// </summary>
        /// <param name="rootBone">Root bone in hierarhy, from that animation will be applied. In Blitz3D you may find bone by FindChild() command and animate it. In Xors3D you must simply pass its name in xAnimate() command. For hierarhy a new animation track will be created, so then you will need to get its info (like time, speed, etc) you must also pass a name of its root bone for correct results</param>     </param>
        /// <returns></returns>
        public float GetAnimLength(string rootBone = "")
        {
            return Xors3d.AnimLength(_handle, rootBone);
        }

        /// <summary>
        /// Sets entity's animation time
        /// </summary>
        /// <param name="time">New animation time </param>
        /// <param name="seq">Animation sequence number </param>
        /// <param name="rootBone">Root bone. See xAnimate() for more infromation </param>
        public void SetAnimTime(float time, int seq, string rootBone = "")
        {
            Xors3d.SetAnimTime(_handle, time, seq, rootBone);
        }

        /// <summary>
        /// Sets entity's animation frame. 
        /// This work similarly with xSetAnimTime(), but operate with frames 
        /// </summary>
        /// <param name="frame">New animation frame </param>
        /// <param name="seq">Animation sequence number </param>
        /// <param name="rootBone">Root bone. See xAnimate() for more infromation </param>
        public void SetAnimFrame(float frame, int seq, string rootBone = "")
        {
            Xors3d.SetAnimFrame(_handle, frame, seq, rootBone);
        }
    }
}
