﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Buffer
    {
        internal int _handle = 0;

        public bool equals(Buffer value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Locks specified buffer. 
        /// You must UnlockBuffer before using other graphics commands or API calls, and you are advised to only keep the buffer locked for as long as it is needed
        /// </summary>
        public void Lock()
        {
            Xors3d.LockBuffer(_handle);
        }

        /// <summary>
        /// Unlocks locked buffer
        /// </summary>
        public void Unlock()
        {
            Xors3d.UnlockBuffer(_handle);
        }

        /// <summary>
        /// Writes a color value to either the current buffer or the specified buffer. 
        /// You must use this command on a locked buffer, otherwise the command will fail 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="argb"></param>
        public void WritePixelFast(int x, int y, int argb)
        {
            Xors3d.WritePixelFast(x, y, argb, _handle);
        }

        /// <summary>
        /// Writes a color value to either the current buffer or the specified buffer. 
        /// You can use this command on a locked buffer for a slight speed-up
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="argb"></param>
        public void WritePixel(int x, int y, int argb)
        {
            Xors3d.WritePixel(x, y, argb, _handle);
        }

        /// <summary>
        /// Reads a color value from either the current buffer or the specified buffer. 
        /// The returned colour value is in the form of an integer that contains the alpha, red, green and blue values of the pixel. You must use this command on a locked buffer, otherwise the command will fail. See xLockBuffer
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public int ReadPixelFast(int x, int y)
        {
            return Xors3d.ReadPixelFast(x, y, _handle);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public int ReadPixel(int x, int y)
        {
            return Xors3d.ReadPixel(x, y, _handle);
        }

        /// <summary>
        /// Reads pixels data either the current buffer or the specified buffer. 
        /// This command must be used with locked buffers, pixels data losts after buffer unlock
        /// </summary>
        /// <returns></returns>
        public int GetPixels()
        {
            return Xors3d.GetPixels(_handle);
        }

        /// <summary>
        /// Save buffer into file. 
        /// Typically, this is used to take a screen snapshot. This will save the spcified buffer you specify to a .bmp file you specify
        /// </summary>
        /// <param name="path"></param>
        public void Save(string path)
        {
            Xors3d.SaveBuffer(_handle, path);
        }

        /// <summary>
        /// Loads and image into specified buffer. 
        /// There are a hundred and one uses for this command, but probably most often used would be to display a title screen or some other 'one time viewing only' image to the back buffer 
        /// </summary>
        /// <param name="path"></param>
        public void Load(string path)
        {
            Xors3d.LoadBuffer(_handle, path);
        }

        public int GetWidth()
        {
            return Xors3d.BufferWidth(_handle);
        }

        public int GetHeight()
        {
            return Xors3d.BufferHeight(_handle);
        }

        /// <summary>
        /// Copys a pixel from one buffer to other
        /// </summary>
        /// <param name="sx">x-coordinate of source pixel to copy from </param>
        /// <param name="sy">y-coordinate of source pixel to copy from </param>
        /// <param name="destBuf">Destination buffer </param>
        /// <param name="dx">x-coordinate of destination pixel to copy to </param>
        /// <param name="dy">y-coordinate of destination pixel to copy to </param>
        public void CopyPixel(int sx, int sy, Buffer destBuf, int dx, int dy)
        {
            Xors3d.CopyPixel(sx, sy, _handle, dx, dy, destBuf._handle);
        }


        /// <summary>
        /// Copys a pixel from one buffer to other
        /// You must use this command on a locked buffer, otherwise the command 
        /// </summary>
        /// <param name="sx">x-coordinate of source pixel to copy from </param>
        /// <param name="sy">y-coordinate of source pixel to copy from </param>
        /// <param name="destBuf">Destination buffer </param>
        /// <param name="dx">x-coordinate of destination pixel to copy to </param>
        /// <param name="dy">y-coordinate of destination pixel to copy to </param>
        public void CopyPixelFast(int sx, int sy, Buffer destBuf, int dx, int dy)
        {
            Xors3d.CopyPixelFast(sx, sy, _handle, dx, dy, destBuf._handle);
        }

        /// <summary>
        /// Copies a rectangle of graphics from one buffer to another. 
        /// If a buffer is omitted, the current buffer is used 
        /// </summary>
        /// <param name="sx"></param>
        /// <param name="sy"></param>
        /// <param name="sw"></param>
        /// <param name="sh"></param>
        /// <param name="destBuf"></param>
        /// <param name="dx"></param>
        /// <param name="dy"></param>
        public void CopyRect(int sx, int sy, int sw, int sh, Buffer destBuf, int dx, int dy)
        {
            Xors3d.CopyRect(sx, sy, sw, sh, dx, dy, _handle, destBuf._handle);
        }



    }
}
