﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Camera : Entity
    {
        private Fog _Fog = null;

        public Fog Fog
        {
            get
            {
                return _Fog;
            }
        }

        public Camera()
        {
            _Fog = new Fog();
            _Fog._camera = this;
        }

        /// <summary>
        /// Sets camera background color. Defaults to 0, 0, 0
        /// </summary>
        /// <param name="color"></param>
        public void SetClsColor(Color color)
        {
            Xors3d.CameraClsColor(_handle, color._red, color._green, color._blue);
        }

        /// <summary>
        /// Sets the camera projection mode
        /// </summary>
        /// <param name="mode"></param>
        public void SetProjMode(ProjectionMode mode)
        {
            Xors3d.CameraProjMode(_handle, (int)mode);
        }

        /// <summary>
        /// Sets camera clear mode
        /// </summary>
        /// <param name="clearColor"></param>
        /// <param name="clearZBuffer"></param>
        public void ClsMode(bool clearColor, bool clearZBuffer)
        {
            Xors3d.CameraClsMode(_handle, clearColor ? 1 : 0, clearZBuffer ? 1 : 0);
        }

        /// <summary>
        /// Returns true if sphere passed frustum culling check
        /// </summary>
        /// <param name="vec"></param>
        /// <param name="radii"></param>
        /// <returns></returns>
        public bool SphereInFrustum(Vector3 vec, float radii)
        {
            return Xors3d.SphereInFrustum(_handle, vec._x, vec._y, vec._z, radii) == 1;
        }

        /// <summary>
        /// Sets camera clip plane. 
        /// (A, B, C, D) coefficients take the form of the general plane equation
        /// </summary>
        /// <param name="index">Clip plane index </param>
        /// <param name="enabled">True to enable clipping plane </param>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <param name="c"></param>
        /// <param name="d"></param>
        public void SetClipPlane(int index, bool enabled, float a, float b, float c, float d)
        {
            Xors3d.CameraClipPlane(_handle, index, enabled, a, b, c, d);
        }

        /// <summary>
        /// Sets camera range. 
        /// Try and keep the ratio of far/near as small as possible for optimal z-buffer performance. Defaults to 1, 1000 
        /// </summary>
        /// <param name="near">Distance in front of camera that 3D objects start being drawn </param>
        /// <param name="range">Distance in front of camera that 3D object stop being drawn </param>
        public void SetRange(float near, float range)
        {
            Xors3d.CameraRange(_handle, near, range);
        }

        /// <summary>
        /// Sets the camera viewport position and size.
        /// The camera viewport is the area of the 2D screen that the 3D graphics as viewed by the camera are displayed in. Setting the camera viewport allows you to achieve spilt-screen and rear-view mirror effects 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        public void SetViewPort(int x, int y, int width, int height)
        {
            Xors3d.CameraViewport(_handle, x, y, width, height);
        }

        /// <summary>
        /// Sets the camera viewport crop rectangle </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        public void SetCropViewPort(int x, int y, int width, int height)
        {
            Xors3d.CameraCropViewport(_handle, x, y, width, height);
        }

        /// <summary>
        /// Projects the world coordinates on to the 2D screen. 
        /// If the point (x, y, z) is behind the camera, projected coordinates are (-65535.0, -65535.0, -65535.0). 
        /// </summary>
        /// <param name="vec"></param>
        public Vector3 Project
        {
            set
            {
                Xors3d.CameraProject(_handle, value._x, value._y, value._z);
            }
            get
            {
                Vector3 v = new Vector3(Xors3d.ProjectedX(), Xors3d.ProjectedY(), Xors3d.ProjectedZ());
                return v;
            }
        }

        /// <summary>
        /// Projects the 2D screen coordinates on to the world coordinates
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="zDistance">Distance of the projected point from camera near plane </param>
        public void Project2D(int x, int y, float zDistance)
        {
            Xors3d.CameraProject2D(_handle, x, y, zDistance);
        }

        /// <summary>
        /// Returns a pointer to camera's view matrix.
        /// </summary>
        /// <returns></returns>
        public D3DXMATRIX GetViewMatrix()
        {
            D3DXMATRIX m = new D3DXMATRIX();
            m._handle = Xors3d.GetViewMatrix(_handle);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Returns a pointer to camera's projection matrix.
        /// </summary>
        /// <returns></returns>
        public D3DXMATRIX GetProjectionMatrix()
        {
            D3DXMATRIX m = new D3DXMATRIX();
            m._handle = Xors3d.GetProjectionMatrix(_handle);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Returns a pointer to camera's vewproj matrix.
        /// </summary>
        /// <returns></returns>
        public D3DXMATRIX GetViewProjMatrix()
        {
            D3DXMATRIX m = new D3DXMATRIX();
            m._handle = Xors3d.GetViewProjMatrix(_handle);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Sets zoom factor for a camera
        /// </summary>
        /// <param name="zoom"></param>
        public void SetZoom(float zoom)
        {
            Xors3d.CameraZoom(_handle, zoom);
        }

        /// <summary>
        /// Returns true if the specified entity is visible to the specified camera. 
        /// If the entity is a mesh, its bounding box will be checked for visibility. For all other types of entities, only their centre position will be checked 
        /// </summary>
        /// <param name="ent"></param>
        /// <returns></returns>
        public bool EntityInView(Entity ent)
        {
            return Xors3d.EntityInView(ent._handle, _handle)==1;
        }

        /// <summary>
        /// Picks the entity positioned at the specified viewport coordinates. 
        /// Returns the entity picked, or 0 if none there. An entity must have its xEntityPickMode() set to a non-0 value value to be 'pickable'
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public Entity Pick(int x, int y)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CameraPick(_handle, x, y);
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Enables shadow rendering for the this camera
        /// </summary>
        public bool EnableShadows
        {
            set
            {
                if (value)
                {
                    Xors3d.CameraEnableShadows(_handle);
                }
                else
                {
                    Xors3d.CameraDisableShadows(_handle);
                }
            }
        }
    }
}
