﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{  
    public class Collision
    {
        internal Entity _entity = null;

        /// <summary>
        /// Returns entity collision type. 
        /// </summary>
        /// <returns>Custom type</returns>
        public int GetEntityType()
        {
            return Xors3d.GetEntityType(_entity._handle);
        }

        /// <summary>
        /// Sets the collision type for an entity. A collision type value of 0 indicates that no collision checking will occur with that entity. A collision value of [1; 999] will mean collision checking will occur. 
        /// </summary>
        /// <param name="typeID">Collision type of entity. Must be in the range [0; 999]</param>
        /// <param name="isRecurse">True to apply collision type to entity's children </param>
        public void SetEntityType(int typeID, bool isRecurse = true)
        {
            Xors3d.EntityType(_entity._handle, typeID, isRecurse);
        }

        /// <summary>
        /// Resets the collision state of an entity
        /// </summary>
        public void ResetEntity()
        {
            Xors3d.ResetEntity(_entity._handle);
        }

        /// <summary>
        /// Sets the radius of an entity's collision sphere. An entity radius should be set for all entities involved in spherical collisions, which is all source entities (as collisions are always sphere-to-something), and whatever destination entities are involved in sphere-to-sphere collisions 
        /// </summary>
        /// <param name="xRadius">x radius of entity's collision sphere </param>
        /// <param name="yRadius">y radius of entity's collision sphere </param>
        public void EntityRadius(float xRadius, float yRadius = 0)
        {
            Xors3d.EntityRadius(_entity._handle, xRadius, yRadius);
        }

        /// <summary>
        /// Sets the dimensions of an entity's collision box. 
        /// </summary>
        /// <param name="entity">Entity </param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="z"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="depth"></param>
        public void xEntityBox(float x, float y, float z, float width, float height, float depth)
        {
            Xors3d.EntityBox(_entity._handle, x, y, z, width, height, depth);
        }

        /// <summary>
        /// Returns true if an entity collided with any other entity of the specified type. 
        /// </summary>
        /// <param name="typeID">Type of entity  </param>
        /// <returns></returns>
        public Entity EntityCollided(int typeID)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.EntityCollided(_entity._handle, typeID);
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Returns how many collisions an entity was involved in during the last xUpdateWorld() 
        /// </summary>
        /// <returns></returns>
        public int CountCollisions()
        {
            return Xors3d.CountCollisions(_entity._handle);
        }

        /// <summary>
        /// Returns the world xyz coordinate of a particular collision. Index should be in the range [0; xCountCollisions(entity) - 1] 
        /// </summary>
        /// <param name="index">Index of collision  </param>
        /// <returns></returns>
        public Vector3 CollisionParticular(int index)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.CollisionX(_entity._handle, index);
            v._y = Xors3d.CollisionY(_entity._handle, index);
            v._z = Xors3d.CollisionZ(_entity._handle, index);
            return v;
        }

        /// <summary>
        /// Returns the xyz component of the normal of a particular collision. Index should be in the range [0; xCountCollisions(entity) - 1] 
        /// </summary>
        /// <param name="index">Index of collision  </param>
        /// <returns></returns>
        public Vector3 CollisionNormalParticular(int index)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.CollisionNX(_entity._handle, index);
            v._y = Xors3d.CollisionNY(_entity._handle, index);
            v._z = Xors3d.CollisionNZ(_entity._handle, index);
            return v;
        }

        /// <summary>
        /// Returns the time taken to calculate a particular collision. Index should be in the range [0; xCountCollisions(entity) - 1] 
        /// </summary>
        /// <param name="index">Index of collision  </param>
        /// <returns></returns>
        public float CollisionTime(int index)
        {
            return Xors3d.CollisionTime(_entity._handle, index);
        }

        /// <summary>
        /// Returns the other entity involved in a particular collision. Index should be in the range [0; xCountCollisions(entity) - 1] 
        /// </summary>
        /// <param name="index">Index of collision </param>
        /// <returns></returns>
        public Entity CollisionEntity(int index)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CollisionEntity(_entity._handle, index);
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Returns the handle of the surface belonging to the specified entity that was closest to the point of a particular collision. Index should be in the range [0; xCountCollisions(entity) - 1] 
        /// </summary>
        /// <param name="index">Index of collision  </param>
        /// <returns></returns>
        public Surface CollisionSurface(int index)
        {
            Surface surface = new Surface();
            surface._handle = Xors3d.CollisionSurface(_entity._handle, index);
            return surface._handle > 0 ? surface : null;
        }

        /// <summary>
        /// Returns the index number of the triangle belonging to the specified entity that was closest to the point of a particular collision. Index should be in the range [0; xCountCollisions(entity) - 1]
        /// </summary>
        /// <param name="index">Index of collision  </param>
        /// <returns></returns>
        public int CollisionTriangle(int index)
        {
            return Xors3d.CollisionTriangle(_entity._handle, index);
        }

        /// <summary>
        /// Sets the pick mode for an entity. 
        /// </summary>
        /// <param name="mode">Entity picking mode. See 'TPickingMode' for more infromation </param>
        /// <param name="obscurer">True to determine that the entity 'obscures' other entities during an xEntityVisible() call.</param>
        /// <param name="recursive">Pick mode will automatically applied to all entity childs </param>
        public void EntityPickMode(PickMode mode, bool obscurer = true, bool recursive = true)
        {
            Xors3d.EntityPickMode(_entity._handle, (int)mode, obscurer, recursive);
        }

    }
}
