﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Control
    {
        internal Entity _entity = null;
        internal bool _visible = true;
        
        /// <summary>
        /// Enables auto fading for an entity. This will cause an entity's alpha level to be adjusted at distances between near and far to create a 'fade-in' effect.
        /// </summary>
        /// <param name="nearRange">Distance in front of the camera at which entity's will start being faded </param>
        /// <param name="farRange">Distance in front of the camera at which entity's will stop being faded (and will be invisible).  </param>
        public void AutoFade(float nearRange, float farRange)
        {
            Xors3d.EntityAutoFade(_entity._handle, nearRange, farRange);
        }

        /// <summary>
        /// Sets the drawing order for an entity. An order value of 0 will mean the entity is drawn normally. A value greater than 0 will mean that entity is drawn first, behind everything else. A value less than 0 will mean the entity is drawn last, in front of everything else. Setting an entity's order to non-0 also disables z-buffering for the entity, so should be only used for simple, convex entities like skyboxes, sprites etc. xEntityOrder() affects the specified entity but none of its child entities, if any exist. 
        /// </summary>
        /// <param name="order">Order that entity will be drawn in</param>
        public void Order(int order)
        {
            Xors3d.EntityOrder(_entity._handle, order);
        }

        /// <summary>
        /// Paints a entity with a brush. The reason for using xPaintEntity() to apply specific properties to a entity using a brush rather than just using xEntityTexture(), xEntityColor(), xEntityShininess() etc, is that you can pre-define one brush, and then paint entities over and over again using just the one command rather than lots of separate ones. 
        /// </summary>
        /// <param name="brush">Brush handle</param>
        public void Paint(Brush brush)
        {
            Xors3d.PaintEntity(_entity._handle, brush._handle);
        }

        /// <summary>
        /// Applies a texture to an entity. The optional frame parameter specifies which texture animation frame should be used as the texture. The optional index parameter specifies which index number should be assigned to the texture. Index numbers are used for the purpose of multitexturing. See xTextureBlend(). 
        /// </summary>
        /// <param name="texture">Texture </param>
        /// <param name="frame">Frame of texture</param>
        /// <param name="index">Index number of texture. Should be in the range [0; 7]. </param>
        /// <param name="isRecursive">If (true) then entity's children will receive the texture too.  </param>
        public void SetTexture(Texture texture, int frame = 0, int index = 0, bool isRecursive = true)
        {
            Xors3d.EntityTexture(_entity._handle, texture._handle, frame, index, isRecursive ? 1 : 0);
        }

        /// <summary>
        /// Attaches an entity to a parent. Parent may be 0, in which case the entity will have no parent 
        /// </summary>
        /// <param name="parent">Parent entity </param>
        /// <param name="isGlobal">True for the child entity to retain its global position and orientation </param>
        public void SetParent(Entity parent = null, bool isGlobal = false)
        {
            Xors3d.EntityParent(_entity._handle, (parent != null ? parent._handle : 0), isGlobal);
        }

        /// <summary>
        /// Shows an entity. Once an entity has been hidden using xHideEntity(), use show entity to make it visible and involved in collisions again. Entities are shown by default after creating/loading them, so you should only need to use xShowEntity() after using xHideEntity(). 
        /// </summary>
        public void Show()
        {
            Xors3d.ShowEntity(_entity._handle);
        }

        /// <summary>
        /// Hides an entity, so that it is no longer visible, and is no longer involved in collisions. The main purpose of hide entity is to allow you to create entities at the beginning of a program, hide them, then copy them and show as necessary in the main game. This is more efficient than creating entities mid-game. If you wish to hide an entity so that it is no longer visible but still involved in collisions, then use EntityAlpha 0 instead. This will make an entity completely transparent. xHideEntity() affects the specified entity and all of its child entities, if any exist. 
        /// </summary>
        public void Hide()
        {
            Xors3d.HideEntity(_entity._handle);
        }

        /// <summary>
        /// set Visible or Invisible mode
        /// </summary>
        public bool Visible
        {
            set
            {
                if (value) Show();
                else Hide();
                _visible = value;
            }
            get
            {
                return _visible;
            }
        }

        /// <summary>
        /// Sets the specular color of an entity. The red, green and blue values should be in the range [0; 255] with 0 being darkest and 255 brightest. The default specular entity color is 0, 0, 0 (black). xEntityShininess() also change specular color. 
        /// </summary>
        /// <param name="color">value of entity specular color </param>
        public void SpecularColor(Color color)
        {
            Xors3d.EntitySpecularColor(_entity._handle,color.Red,color.Green,color.Blue);
        }

        /// <summary>
        /// Sets the ambient color of an entity. The red, green and blue values should be in the range [0; 255] with 0 being darkest and 255 brightest. The default entity ambient color is 255, 255, 255 (white). xEntityColor() also change ambient color 
        /// </summary>
        /// <param name="color"></param>
        public void AmbientColor(Color color)
        {
            Xors3d.EntityAmbientColor(_entity._handle, color.Red, color.Green, color.Blue);
        }

        public void EmissiveColor(Color color)
        {
            Xors3d.EntityEmissiveColor(_entity._handle, color.Red, color.Green, color.Blue);
        }

    }
}
