﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xors.Physics;

namespace Xors
{
    public class Entity
    {
        internal int _handle = 0;
        internal bool _bump = false;
        internal Camera _bump_camera = null;

        private Movement _Movement = new Movement();
        private State _State = new State();
        private Control _Control = new Control();
        private Collision _Collision = new Collision();
        private PhysicsEntity _Physics = new PhysicsEntity();
        private Joints _Joints = new Joints();
        private ShaderEffectsEntity _ShaderEffect = new ShaderEffectsEntity();

        /// <summary>
        /// returned Xors3D handle
        /// </summary>
        /// <returns></returns>
        public int GetHandle()
        {
            return _handle;
        }

        /// <summary>
        /// Adds behaviour to entity
        /// For update call XE.UpdateBehaviours();
        /// </summary>
        /// <param name="behav"></param>
        public void AddBehaviour(Behaviour behav)
        {
            behav.self = this;
            behav.OnCreate();
            XE.behavs.Add(behav);
        }

        /// <summary>
        /// Matches the objects handles to each other
        /// </summary>
        /// <param name="second"></param>
        /// <returns></returns>
        public bool equals(Entity second)
        {
            try
            {
                if (_handle == second._handle) return true;
                else return false;
            }
            catch(Exception ex)
            {
                return false;
            }
        }

        /// <summary>
        /// Enables or disables shadow casting for an entity. 
        /// In some cases you need some entity not casting the shadows from a specified light source (e.g. light bulb). This function excludes the entity from the list of shadow casters. By default entity casts shadows from all light sources.
        /// Note: Any transparent or ordered entity is automtically excluded from the list of casters
        /// </summary>
        /// <param name="light">Light source from which the entity will not cast shadows. Set 1 to affect all the lights</param>
        /// <param name="state">True to make the entity cast shadows, false for otherwise. </param>
        public void ShadowsCast(Light light, bool state)
        {
            Xors3d.EntityCastShadows(_handle, light._handle, state);
        }

        /// <summary>
        /// Checks if an entity casts shadows from the light source
        /// </summary>
        /// <param name="light"></param>
        /// <returns></returns>
        public bool isCasterShadow(Light light)
        {
            return Xors3d.EntityIsCaster(_handle, light._handle)==1;
        }

        /// <summary>
        /// Enables or disables shadow receiving for an entity. 
        /// In some cases you need some entity not receiving the shadows from a specified light source (e.g. fullbrighted entity). This function excludes the entity from the list of shadow receivers. By default entity receives shadows from all light sources.
        /// Note: Any transparent or ordered entity is automtically excluded from the list of receivers
        /// </summary>
        /// <param name="light"></param>
        /// <param name="state"></param>
        public void ShadowsReceive(Light light, bool state)
        {
            Xors3d.EntityReceiveShadows(_handle, light._handle, state);
        }

        /// <summary>
        /// Checks if entity receive shadows from the light source
        /// </summary>
        /// <param name="light"></param>
        /// <returns></returns>
        public bool isReceiverShadow(Light light)
        {
            return Xors3d.EntityIsReceiver(_handle, light._handle) == 1;
        }

        

        public Entity()
        {
            _Movement._entity = this;
            _State._entity = this;
            _Control._entity = this;
            _Collision._entity = this;
            _Physics._entity = this;
            _Joints._entity = this;
            _ShaderEffect._entity = this;
        }

        /// <summary>
        /// Creates entity instancee and returns its handle. Instncing allows rendering multiple copies of the same mesh in a scene at once. This technique is primarily used for objects such as trees, grass, or buildings which can be represented as repeated geometry without appearing unduly repetitive, but may also be used for characters.You may rotate, move and scale instance or change its color and alpha value. 
        /// </summary>
        /// <param name="parent">Parent entity </param>
        /// <returns></returns>
        public Entity CreateInstance(Entity parent)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CreateInstance(_handle, parent._handle);
            if (ent._handle > 0) return ent;
            else return null;
        }

        /// <summary>
        /// Frees up an entity's handle. 
        /// </summary>
        public void FreeEntity()
        {
            Xors3d.FreeEntity(_handle);
        }

        /// <summary>
        /// Creates a copy of an entity and returns the handle of the newly created copy. This is a new entity instance of an existing entity's mesh. Anything you do to the original mesh (such as xRotateMesh()) will effect all the copies. Other properties (such as xEntityColor(), xPositionEntity(), etc.) since they are 'Entity' properties, will be individual to the copy. If a parent entity is specified, the copied entity will be created at the parent entity's position. Otherwise, it will be created at 0, 0, 0. 
        /// </summary>
        /// <param name="parent">Parent entity </param>
        /// <param name="cloneBuffers">If true vertex and indices buffers will be cloned </param>
        /// <returns></returns>
        public Entity CopyEntity(Entity parent = null, bool cloneBuffers = false)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CopyEntity(_handle, parent._handle, cloneBuffers ? 1 : 0);
            if (ent._handle > 0) return ent;
            else return null;
        }

        public Movement Movement
        {
            get { return _Movement; }
        }

        public State State
        {
            get { return _State; }
        }

        public Control Control
        {
            get { return _Control; }
        }

        public Collision Collision
        {
            get { return _Collision; }
        }

        public PhysicsEntity Physics
        {
            get { return _Physics; }
        }

        public Joints Joints
        {
            get { return _Joints; }
        }

        public ShaderEffectsEntity ShaderEffect
        {
            get { return _ShaderEffect; }
        }
    }
}
