﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class FileSystem
    {

        /// <summary>
        /// Mounts a zip file as a part of the file system and returns its handle. 
        /// Use a zip file as a pack of your resources. Use of packs allows to compress the resources and to encrypt them with a password. Once mounted pack file becomes visible for the engine as a part of the file system. The second parameter is a mount point which is the location in the file system structure where a mounted pack file appears
        /// When you load some media, the engine searches that media in the mounted pack files first. For instance, the pack was mounted by using following
        /// xMountPackFile("textures.zip", "textures\", "super_password");
        /// Since now a pack file "textures.zip" is seen as a "textures\" directory.
        /// you load some texture by using
        /// xLoadTexture("textures\something.dds");
        /// the engine will try to load a texture "something.dds" from the pack mounted before since its mount point is "textures\". If there isn't such texture in the pack, the engine will try to load it from the disk using the path "textures\something.dds".
        /// Such system allows you to develop and release your applications with ease. During development use loading from the disk. Before release pack your media and mount the packs using proper mount points.
        /// Note: With packfiles works only xReadFile(), xFileType() and xFileSize() commands. You can't create, delete or change files in packfiles.
        /// Parameters:
        /// </summary>
        /// <param name="path">A path to the pack file</param>
        /// <param name="mountPoint">Mount point of the pack file</param>
        /// <param name="pass"> password of the pack file</param>
        /// <returns></returns>
        public static PackFile MountPackFile(string path, string mountPoint = "", string pass = "")
        {
            PackFile p = new PackFile();
            p._handle = Xors3d.MountPackFile(path, mountPoint, pass);
            return p._handle > 0 ? p : null;
        }

        /// <summary>
        /// Opens the designated filename and prepares it to be written to
        /// </summary>
        /// <param name="path">Filename to opens </param>
        /// <returns></returns>
        public static File WriteFile(string path)
        {
            File fil = new File();
            fil._handle = Xors3d.WriteFile(path);
            return fil._handle > 0 ? fil : null;
        }

        /// <summary>
        /// Checks the filename you pass and determines if it exists and whether or not it is a valid filename or if it is a director
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static FileType GetFileType(string path)
        {
            return (FileType)Xors3d.FileType(path);
        }

        /// <summary>
        /// Returns the size of a file
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static int GetFileSize(string path)
        {
            return GetFileSize(path);
        }

        /// <summary>
        /// Returns a directory handle. You will use the xNextFile() to iterate through each file entry (use xFileType() to see if it is a file or folder). 
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static Dir ReadDir(string path)
        {
            Dir dir = new Dir();
            dir._handle = Xors3d.ReadDir(path);
            return dir._handle>0?dir:null;
        }

        /// <summary>
        /// Returns current directory path
        /// </summary>
        /// <returns></returns>
        public static string CurrentDir()
        {
            return Xors3d.CurrentDir();
        }

        /// <summary>
        /// Sets directory as current
        /// </summary>
        /// <param name="path"></param>
        public static void ChangeDir(string path)
        {
            Xors3d.ChangeDir(path);
        }

        /// <summary>
        /// Creates new directory
        /// </summary>
        /// <param name="path"></param>
        public static bool CreateDir(string path)
        {
            return Xors3d.CreateDir(path)==1?true:false;
        }

        /// <summary>
        /// Delete directory
        /// </summary>
        /// <param name="path"></param>
        public static bool DeleteDir(string path)
        {
            return Xors3d.DeleteDir(path) == 1 ? true : false;
        }

        /// <summary>
        /// Copies file from source path to destination
        /// </summary>
        /// <param name="src"></param>
        /// <param name="dest"></param>
        /// <returns></returns>
        public static bool CopyFile(string src, string dest)
        {
            return Xors3d.CopyFile(src, dest) == 1 ? true : false;
        }

        /// <summary>
        /// Delete file
        /// </summary>
        /// <param name="path"></param>
        public static bool DeleteFile(string path)
        {
            return Xors3d.DeleteFile(path) == 1 ? true : false;
        }
    }
}
