﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class Logging
    {
        /// <summary>
        /// Starts logging. 
        /// This function enables engine logging. You can specify logging level, target, html-file and css-file for log output
        /// </summary>
        /// <param name="target">Logging target. See 'Logging output targets' for more information </param>
        /// <param name="level">Logging level. See 'Logging level' for more information </param>
        /// <param name="filename">Log file for output (needed if HTML logging is enabled). Existing file will be overwritten</param>
        /// <param name="cssfilename">path to an external style sheet used by HTML log. A full or relative path is acceptable</param>
        public static void CreateLog(LoggingTarget target = LoggingTarget.HTML, LoggingLevel level = LoggingLevel.INFO, string filename = "xors_log.html", string cssfilename = "")
        {
            Xors3d.CreateLog((int)target, (int)level, filename, cssfilename);
        }

        /// <summary>
        /// Finishes logging. 
        /// This function finishes logging and closes log file (if HTML output was enabled). 
        /// </summary>
        /// <returns></returns>
        public static int Close()
        {
            return Xors3d.CloseLog();
        }

        /// <summary>
        /// Returns string which contains log messages. 
        /// This function returns log meessages written since last xClearLogString() or xGetLogString() call. 
        /// </summary>
        /// <returns></returns>
        public static string GetLogString()
        {
            return Xors3d.GetLogString();
        }

        /// <summary>
        /// Clears log string. 
        /// Clears string which contains log messages. This command doesn't clear html log-file or standard output stream (std::cout). 
        /// </summary>
        public static void ClearLogString()
        {
            Xors3d.ClearLogString();
        }

        /// <summary>
        /// Changes logging level. 
        /// Logging level controls types of messages to be written to log. See 'Logging level' for more information about logging levels
        /// </summary>
        /// <param name="level"></param>
        public static LoggingLevel LogLevel
        {
            set
            {
                Xors3d.SetLogLevel((int)value);
            }
            get
            {
                return (LoggingLevel)Xors3d.GetLogLevel();
            }
        }

        /// <summary>
        /// Changes logging target. 
        /// This function allows you to specify where log messages should be written. See 'Logging output targets' for more information about logging targets
        /// </summary>
        public static LoggingTarget LogTarget
        {
            set
            {
                Xors3d.SetLogTarget((int)value);
            }
            get
            {
                return (LoggingTarget)Xors3d.GetLogTarget();
            }
        }

        /// <summary>
        /// Adds a Info-styled entry to log system. 
        /// This function is useful for making log entries from your own code and watching them along with entries made by Xors3d core.
        /// Func, file and line parameters are displayed in hovering window in HTML format. If func and file are empty and line is -1, the hovering window won't be displayed
        /// </summary>
        /// <param name="message">The text of entry to be logged </param>
        /// <param name="func">Function name </param>
        /// <param name="file">File name </param>
        /// <param name="line">Line of code </param>
        public static void SetLogInfo(string message, string func = "", string file = "", int line = -1)
        {
            Xors3d.LogInfo(message, func, file, line);
        }

        public static void SetLogMessage(string message, string func = "", string file = "", int line = -1)
        {
            Xors3d.LogMessage(message, func, file, line);
        }

        public static void SetLogWarning(string message, string func = "", string file = "", int line = -1)
        {
            Xors3d.LogWarning(message, func, file, line);
        }

        public static void SetLogError(string message, string func = "", string file = "", int line = -1)
        {
            Xors3d.LogError(message, func, file, line);
        }

        public static void SetLogFatal(string message, string func = "", string file = "", int line = -1)
        {
            Xors3d.LogFatal(message, func, file, line);
        }

    }
}
