﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Mesh : Entity
    {
        /// <summary>
        /// Creates a copy of a mesh and returns the newly-created mesh's handle. 
        /// The difference between xCopyMesh() and xCopyEntity() is that xCopyMesh() performs a 'deep' copy of a mesh
        /// </summary>
        /// <param name="parent"></param>
        /// <returns></returns>
        public Mesh CopyMesh(Entity parent=null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.CopyMesh(_handle, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Flips all the triangles in a mesh. 
        /// This is useful for a couple of reasons. Firstly though, it is important to understand a little bit of the theory behind 3D graphics. A 3D triangle is represented by three points; only when these points are presented to the viewer in a clockwise-fashion is the triangle visible. So really, triangles only have one side. Normally, for example in the case of a sphere, a model's triangles face the inside of the model, so it doesn't matter that you can't see them. However, what about if you wanted to use the sphere as a huge sky for your world, i.e. so you only needed to see the inside? In this case you would just use xFlipMesh(). Another use for xFlipMesh() is to make objects two-sided, so you can see them from the inside and outside if you can't already. In this case, you can copy the original mesh using xCopyEntity(), specifying the original mesh as the parent, and flip it using xFlipMesh(). You will now have two meshes occupying the same space - this will make it double-sided, but beware, it will also double the polygon count. The above technique is worth trying when an external modelling program has exported a model in such a way that some of the triangles appear to be missing. 
        /// </summary>
        public void FlipMesh()
        {
            Xors3d.FlipMesh(_handle);
        }

        /// <summary>
        /// Paints a mesh with a brush. 
        /// This has the effect of instantly altering the visible appearance of the mesh, assuming the brush's properties are different to what was was applied to the surface before. The reason for using xPaintMesh() to apply specific properties to a mesh using a brush rather than just using xEntityTexture(), xEntityColor(), xEntityShininess() etc, is that you can pre-define one brush, and then paint meshes over and over again using just the one command rather than lots of separate ones. 
        /// </summary>
        /// <param name="brush"></param>
        public void Paint(Brush brush)
        {
            Xors3d.PaintMesh(_handle, brush._handle);
        }

        /// <summary>
        /// Scales and translates all vertices of a mesh so that the mesh occupies the specified box. 
        /// Do not use a width, height or depth value of 0.0, otherwise all mesh data will be destroyed and your mesh will not be displayed. Use a value of 0.001 instead for a flat mesh along one axis
        /// </summary>
        /// <param name="vec">position of mesh </param>
        /// <param name="width">Width of mesh </param>
        /// <param name="height">Height of mesh </param>
        /// <param name="depth">Depth of mesh </param>
        /// <param name="uniform">If true, the mesh will be scaled by the same amounts in x, y and z, so will not be distorted </param>
        public void Fit(Vector3 vec, float width, float height, float depth, bool uniform = false)
        {
            Xors3d.FitMesh(_handle, vec._x, vec._y, vec._z, width, height, depth, uniform);
        }

        /// <summary>
        /// Return mesh dimension x = width, y = height, z = depth
        /// </summary>
        /// <param name="recursive">If true - will compute bounding box width for all heirarhy </param>
        /// <returns></returns>
        public Vector3 GetDimension(bool recursive=false)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.MeshWidth(_handle, recursive);
            v._y = Xors3d.MeshHeight(_handle, recursive);
            v._z = Xors3d.MeshDepth(_handle, recursive);
            return v;
        }

        /// <summary>
        /// Scales all vertices of a mesh by the specified scaling factors. 
        /// Unlike xScaleEntity(), xScaleMesh() actually modifies the actual mesh structure. So whereas using xScaleEntity(2, 2, 2) would only double the size of an entity the first time it was used, xScaleMesh(2, 2, 2) will double the size of the mesh every time it is used. This is because xScaleEntity() scales an entity based on a fixed mesh structure, whereas xScaleMesh() actually modifies the mesh structure itself. 
        /// </summary>
        /// <param name="vec"></param>
        public void ScaleMesh(Vector3 vec)
        {
            Xors3d.ScaleMesh(_handle, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Rotates all vertices of a mesh by the specified rotation. 
        /// Unlike xRotateEntity(), xRotateMesh() actually modifies the actual mesh structure. So whereas using xRotateEntity(0, 45, 0) would only rotate an entity by 45 degrees the first time it was used, xRotateMesh(0, 45, 0) will rotate the mesh every time it is used. This is because xRotateEntity() rotates an entity based on a fixed mesh structure, whereas xRotateMesh() actually modifies the mesh structure itself 
        /// </summary>
        /// <param name="vec"></param>
        public void RotateMesh(Vector3 vec)
        {
            Xors3d.RotateMesh(_handle, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Moves all vertices of a mesh. 
        /// Unlike xPositionEntity(), xPositionMesh() modifies the actual mesh structure. So whereas using xPositionEntity(0, 0, 1) would only move an entity by one unit the first time it was used, xPositionMesh(0, 0, 1) will move the mesh by one unit every time it is used. This is because xPositionEntity() positions an entity based on a fixed mesh structure, whereas xPositionMesh() actually modifies the mesh structure itself. 
        /// </summary>
        /// <param name="vec"></param>
        public void PositionMesh(Vector3 vec)
        {
            Xors3d.PositionMesh(_handle, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Recalculates all normals, tangents and binormals in a mesh. 
        /// This is necessary for correct lighting if you have not set surface normals using xVertexNormal() commands
        /// </summary>
        public void UpdateNormals()
        {
            Xors3d.UpdateNormals(_handle);
        }

        /// <summary>
        /// Recalculates all normals in a mesh. 
        /// This is necessary for correct lighting if you have not set surface normals using xVertexNormal() commands
        /// </summary>
        public void UpdateN()
        {
            Xors3d.UpdateN(_handle);
        }
        
        /// <summary>
        /// Recalculates all tangents and binormals in a mesh. 
        /// This is necessary for correct lighting if you have not set surface normals using xVertexBinormal() and xVertexTangent()commands
        /// </summary>
        public void UpdateTB()
        {
            Xors3d.UpdateTB(_handle);
        }

        /// <summary>
        /// Returns true if the specified meshes bounding boxes are currently intersecting. 
        /// Deprecated:
        /// Use xEntitiesBBIntersect() instead. 
        /// </summary>
        /// <param name="entity2">Second entity </param>
        /// <returns></returns>
        public bool MeshBBIntersect(Entity entity2)
        {
            return Xors3d.MeshesBBIntersect(_handle, entity2._handle)==1;
        }

        /// <summary>
        /// Returns true if the specified meshes are currently intersecting
        /// </summary>
        /// <param name="entity2">Second entity</param>
        /// <returns></returns>
        public bool MeshIntersect(Entity entity2)
        {
            return Xors3d.MeshesIntersect(_handle, entity2._handle) == 1;
        }

        /// <summary>
        /// Returns pointer to mesh vertices. 
        /// Actually it's a only copy of original vertex buffer, you can use it for phisics engines, such as PhysX for making trimesh and so one. You must delete created buffers when it's no more needed by xDeteleMeshVB() 
        /// </summary>
        /// <returns></returns>
        public IntPtr GetMeshVB()
        {
            return new IntPtr(Xors3d.GetMeshVB(_handle));
        }

        /// <summary>
        /// Returns a pointer to mesh indeices. 
        /// Actually it's only a copy of original index buffer, you can use it for phisics engines, such as PhysX for making trimesh and so one. You must delete created buffers when it's no more needed by xDeteleMeshIB() 
        /// </summary>
        /// <returns></returns>
        public IntPtr GetMeshIB()
        {
            return new IntPtr(Xors3d.GetMeshIB(_handle));
        }

        public int GetMeshVBSize()
        {
            return Xors3d.GetMeshVBSize(_handle);
        }

        public int GetMeshIBSize()
        {
            return Xors3d.GetMeshIBSize(_handle);
        }

        public void DeleteMeshVB()
        {
            Xors3d.DeleteMeshVB(_handle);
        }

        public void DeleteMeshIB()
        {
            Xors3d.DeleteMeshIB(_handle);
        }

        /// <summary>
        /// Returns the number of surfaces in a mesh. 
        /// Surfaces are sections of mesh. A mesh may contain only one section, or very many 
        /// </summary>
        /// <returns></returns>
        public int GetCountSurfaces()
        {
            return Xors3d.CountSurfaces(_handle);
        }

        /// <summary>
        /// Returns the handle of the surface attached to the specified mesh and with the specified index number. 
        /// Index should be in the range [0; xCountSurfaces() - 1] You need to 'get a surface', i.e. get its handle, in order to be able to then use that particular surface with other commands
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Surface GetSurface(int index)
        {
            Surface s = new Surface();
            s._handle = Xors3d.GetSurface(_handle, index);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Creates a pivot entity. 
        /// A pivot entity is an invisible point in 3D space that's main use is to act as a parent entity to other entities. The pivot can then be used to control lots of entities at once, or act as new centre of rotation for other entities. To enforce this relationship; use xEntityParent() or make use of the optional parent entity parameter available with all entity load/creation commands. Indeed, this parameter is also available with the xCreatePivot() command if you wish for the pivot to have a parent entity itself
        /// </summary>
        /// <param name="parent"></param>
        /// <returns></returns>
        public Entity CreatePivot(Entity parent = null)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CreatePivot(parent != null ? parent._handle : 0);
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Attempts to find a surface attached to the specified mesh and created with the specified brush. 
        /// Returns the surface handle if found or 0 if not. 
        /// </summary>
        /// <param name="brush"></param>
        /// <returns></returns>
        public Surface FindSurface(Brush brush)
        {
            Surface s = new Surface();
            s._handle = Xors3d.FindSurface(_handle, brush._handle);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Creates a polygon
        /// </summary>
        /// <param name="sides">If true two-sided polygon will be created </param>
        /// <param name="parent"></param>
        /// <returns></returns>
        public Entity CreatePoly(bool sides, Entity parent = null)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CreatePoly(sides?1:0, parent != null ? parent._handle : 0);
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Converts an entity with several surfaces into an entity with a single surface. All textures of this entity become ignorable. 
        /// </summary>
        public void MeshSingleSurface()
        {
            Xors3d.MeshSingleSurface(_handle);
        }

        /// <summary>
        /// Saves mesh hierarhy into the .FBX file
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public bool SaveMeshToFBX(string path)
        {
            return Xors3d.SaveMesh(_handle, path)==1;
        }

        /// <summary>
        /// Performs a 'fake' lighting operation on a mesh
        /// </summary>
        /// <param name="color"></param>
        /// <param name="range"></param>
        /// <param name="vec"></param>
        public void SetLightMesh(Color color, Vector3 vec,float range = 0.0f)
        {
            Xors3d.LightMesh(_handle, color._red, color._green, color._blue, range, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Sets the type of primitive to render
        /// </summary>
        /// <param name="type"></param>
        public void SetPrimitiveType(MeshPriminiveType type)
        {
            Xors3d.MeshPrimitiveType(_handle, (int)type);
        }

        /// <summary>
        /// Creates a surface attached to a mesh and returns the surface's handle. 
        /// Surfaces are sections of mesh which are then used to attach triangles to. You must have at least one surface per mesh in order to create a visible mesh, however you can use as many as you like. Splitting a mesh up into lots of sections allows you to affect those sections individually, which can be a lot more useful than if all the surfaces are combined into just one. 
        /// </summary>
        /// <param name="brush"></param>
        /// <param name="dynamic">If true - dynamic surface will be created. It is faster for editing, but slower for rendering (useful for particle systems and others dynamic meshes). </param>
        /// <returns></returns>
        public Surface CreateSurface(Brush brush = null, bool dynamic = false)
        {
            Surface s = new Surface();
            s._handle =  Xors3d.CreateSurface(_handle, brush != null ? brush._handle : 0, dynamic);
            return s._handle > 0 ? s : null;
        }




        public static Mesh CreateCube(Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.CreateCube(parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Creates a 'blank' mesh entity and returns its handle. 
        /// When a mesh is first created it has no surfaces, vertices or triangles associated with it 
        /// </summary>
        /// <param name="parent">Parent entity handle </param>
        /// <returns></returns>
        public static Mesh CreateMesh(Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.CreateMesh(parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Loads a mesh from an .B3D file and returns the mesh handle. 
        /// Any hierarchy and animation information in the file will be ignored. Use xLoadAnimMesh() to maintain hierarchy and animation information. The parent parameter allows you to specify a parent entity for the mesh so that when the parent is moved the child mesh will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the mesh being created at position 0, 0, 0 rather than at the parent entity's position. 
        /// </summary>
        /// <param name="path"></param>
        /// <param name="parent"></param>
        /// <returns></returns>
        public static Mesh LoadMesh(string path, Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.LoadMesh(path, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Loads a static mesh with saving hierarchy
        /// </summary>
        /// <param name="path"></param>
        /// <param name="parent"></param>
        /// <returns></returns>
        public static Mesh LoadMeshWithChild(string path, Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.LoadMeshWithChild(path, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Creates a sphere mesh/entity and returns its handle. 
        /// The sphere will be centred at 0, 0, 0 and will have a radius of 1. The segments value must be in the range 2-100 inclusive
        /// Example segments values:
        /// 8: 224 polygons - bare minimum amount of polygons for a sphere
        /// 16: 960 polygons - smooth looking sphere at medium-high distances
        /// 32: 3968 polygons - smooth sphere at close distances
        /// The parent parameter allows you to specify a parent entity for the sphere so that when the parent is moved the child sphere will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the sphere being created at position 0, 0, 0 rather than at the parent entity's position. 
        /// </summary>
        /// <param name="segments">Sphere detail </param>
        /// <param name="parent"></param>
        /// <returns></returns>
        public static Mesh CreateSphere(int segments = 16, Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.CreateSphere(segments, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Creates a cylinder mesh/entity and returns its handle. 
        /// The cylinder will be centred at 0, 0, 0 and will have a radius of 1. The segments value must be in the range 3-100 inclusive. Example segments values (solid mesh):
        /// 3: 8 polygons - a prism
        /// 8: 28 polygons - bare minimum amount of polygons for a cylinder
        /// 16: 60 polygons - smooth cylinder at medium-high distances
        /// 32: 124 polygons - smooth cylinder at close distances
        /// The optional parent parameter allows you to specify a parent entity for the cylinder so that when the parent is moved the child cylinder will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the cylinder being created at position 0, 0, 0 rather than at the parent entity's position. 
        /// </summary>
        /// <param name="segments">Cylinder detail </param>
        /// <param name="solid">True for a cone with a base, false for a cone without a base </param>
        /// <param name="parent"></param>
        /// <returns></returns>
        public static Mesh CreateCylinder(int segments = 16, bool solid = true, Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.CreateCylinder(segments, solid, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Creates a torus mesh/entity and returns its handle. 
        /// The torus will be centred at 0, 0, 0 and will have a radius of 1. The segments value must be in the range 3-100 inclusive. The optional parent parameter allows you to specify a parent entity for the cylinder so that when the parent is moved the child cylinder will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the torus being created at position 0, 0, 0 rather than at the parent entity's position. 
        /// </summary>
        /// <param name="segments">Torus detail </param>
        /// <param name="R">The distance from the center of the tube to the center of the torus </param>
        /// <param name="r_tube">The radius of the tube </param>
        /// <param name="parent"></param>
        /// <returns></returns>
        public static Mesh CreateTorus(int segments = 16, float R = 1.0f, float r_tube = 0.025f, Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.CreateTorus(segments, R, r_tube, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Creates a cone mesh/entity and returns its handle. 
        /// The cone will be centred at 0, 0, 0 and the base of the cone will have a radius of 1. The segments value must be in the range 3-100 inclusive. Example segments values (solid mesh):
        /// 4: 6 polygons - a pyramid
        /// 8: 14 polygons - bare minimum amount of polygons for a cone
        /// 16: 30 polygons - smooth cone at medium-high distances
        /// 32: 62 polygons - smooth cone at close distances
        /// The optional parent parameter allows you to specify a parent entity for the cone so that when the parent is moved the child cone will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the cone being created at position 0, 0, 0 rather than at the parent entity's position. 
        /// </summary>
        /// <param name="segments"></param>
        /// <param name="solid"></param>
        /// <param name="parent"></param>
        /// <returns></returns>
        public static Mesh CreateCone(int segments = 16, bool solid = true, Entity parent = null)
        {
            Mesh m = new Mesh();
            m._handle = Xors3d.CreateCone(segments, solid, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Adds the source mesh to the destination mesh. 
        /// xAddMesh() works best with meshes that have previously only had mesh commands used with them. So if you want to manipulate a mesh before adding it to another mesh, make sure you use xScaleMesh(), xPositionMesh(), xPaintMesh() etc rather than xScaleEntity(), xPositionEntity(), xEntityTexture() etc before using xAddMesh(). However, something to be aware of when using commands such as xRotateMesh is that all mesh commands work from a global origin of (0, 0, 0). Therefore it is generally a good idea to scale and rotate a mesh before positioning it, otherwise your mesh could end up in unexpected positions. Also, when using xAddMesh(), the origin of the new all-in-one mesh will be set at (0, 0, 0). After using xAddMesh(), the original 'src' mesh will still exist, therefore use xFreeEntity() to delete it if you wish to do so. 
        /// </summary>
        /// <param name="source"></param>
        /// <param name="destination"></param>
        public static void AddMesh(Mesh source, Mesh destination)
        {
            Xors3d.AddMesh(source._handle, destination._handle);
        }
    }
}
