﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Movement
    {
        internal Entity _entity = null;
        /// <summary>
        /// Aligns an entity axis to a vector. 
        /// </summary>
        /// <param name="vector">Vector</param>
        /// <param name="axis">Axis of entity that will be aligned to vector. See 'Axis' for more information </param>
        /// <param name="factor">Rate at which entity is aligned from current orientation to vector orientation. Should be in the range [0.0; 1.0], 0.0 for smooth transition and 1.0 for 'snap' transition. Default is 1.0  </param>
        public void AlignToVector(Vector3 vector, int axis, float factor = 1.0f)
        {
            Xors3d.AlignToVector(_entity._handle, vector.X, vector.Y, vector.Z, axis, factor);
        }

        /// <summary>
        /// Points one entity at another. The roll parameter allows you to specify a roll angle as pointing an entity only sets pitch and yaw angles. If you wish for an entity to point at a certain position rather than another entity, simply create a pivot entity at your desired position, point the entity at this and then free the pivot. 
        /// </summary>
        /// <param name="destination">Destination entity handle </param>
        /// <param name="roll">Roll angle of entity </param>
        public void PointEntity(Entity destination, float roll = 0)
        {
            Xors3d.PointEntity(_entity._handle, destination._handle, roll);
        }


        /// <summary>
        /// Returns xyz-coordinate of the entity. 
        ///If the 'isGlobal' flag is set to ffalse then the parent's local coordinate system is used. NOTE: If the entity has no parent then local and global coordinates are the same. In this case you can think of the 3d world as the parent. Global coordinates refer to the 3d world. Xors3D uses a left-handed system:
        ///X+ is to the right
        ///Y+ is up
        ///Z+ is forward (into the screen)
        ///Every entity also has its own ;ocal coordinate system. The global system never changes. But the local system is carried along as an entity moves and turns. 
        /// </summary>
        /// <param name="glob">True for global coordinates, false for local </param>
        /// <returns></returns>
        public Vector3 GetPosition(bool glob = false)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityX(_entity._handle, glob);
            v._y = Xors3d.EntityY(_entity._handle, glob);
            v._z = Xors3d.EntityZ(_entity._handle, glob);
            return v;
        }

        /// <summary>
        /// Returns xyz-axis scale of entity. 
        /// </summary>
        /// <returns></returns>
        public Vector3 GetScale()
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityScaleX(_entity._handle);
            v._y = Xors3d.EntityScaleY(_entity._handle);
            v._z = Xors3d.EntityScaleZ(_entity._handle);
            return v;
        }

        /// <summary>
        /// Returns the pitch, yaw, roll angle of an entity. 
        /// </summary>
        /// <param name="glob">True if the roll angle returned should be relative to 0 rather than a parent entity's pitch, yaw, roll angle </param>
        /// <returns></returns>
        public Vector3 GetRotation(bool glob = false)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityPitch(_entity._handle, glob);
            v._y = Xors3d.EntityYaw(_entity._handle, glob);
            v._z = Xors3d.EntityRoll(_entity._handle, glob);
            return v;
        }

        
        /// <summary>
        /// Turns an entity relative to its current orientation. 
        /// </summary>
        /// <param name="vec">XYZ Angle</param>
        /// <param name="global">True to turn in global coordinate system, false - for local </param>
        public void Turn(Vector3 vec, bool global = false)
        {
            Xors3d.TurnEntity(_entity._handle, vec.X, vec.Y, vec.Z, global);
        }
        public void Turn(float x, float y, float z, bool global = false)
        {
            Turn(new Vector3(x, y, z), global);
        }

        /// <summary>
        /// Rotates an entity so that it is at an absolute orientation. 
        /// </summary>
        /// <param name="vec">Angle in degrees of pitch, yaw, roll rotation </param>
        /// <param name="global">True if the angle rotated should be relative to 0, 0, 0 rather than a parent entity's orientation  </param>
        public void Rotate(Vector3 vec, bool global = false)
        {
            Xors3d.RotateEntity(_entity._handle, vec.X, vec.Y, vec.Z, global);
        }
        public void Rotate(float x, float y, float z, bool global = false)
        {
            Rotate(new Vector3(x, y, z), global);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="vec"></param>
        /// <param name="global"></param>
        public void Position(Vector3 vec, bool global = false)
        {
            Xors3d.PositionEntity(_entity._handle, vec.X, vec.Y, vec.Z, global);
        }
        public void Position(float x, float y, float z, bool global = false)
        {
            Position(new Vector3(x, y, z), global);
        }


        /// <summary>
        /// Translates an entity relative to its current position and not its orientation.What this means is that an entity will move in a certain direction despite where it may be facing. Imagine that you have a game character that you want to make jump in the air at the same time as doing a triple somersault. Translating the character by a positive y amount will mean the character will always travel directly up in their air, regardless of where it may be facing due to the somersault action. 
        /// </summary>
        /// <param name="vec">XYZ amount that entity will be translated by </param>
        /// <param name="global">Unused </param>
        public void Translate(Vector3 vec, bool global = false)
        {
            Xors3d.TranslateEntity(_entity._handle, vec.X, vec.Y, vec.Z, global);
        }
        public void Translate(float x, float y, float z, bool global = false)
        {
            Translate(new Vector3(x, y, z), global);
        }

        /// <summary>
        /// Positions an entity at an absolute position in 3D space.Entities are positioned using an x, y, z coordinate system. x, y and z each have their own axis, and each axis has its own set of values. By specifying a value for each axis, you can position an entity anywhere in 3D space. 0, 0, 0 is the centre of 3D space, and if the camera is pointing in the default positive z direction, then positioning an entity with a z value of above 0 will make it appear in front of the camera, whereas a negative z value would see it disappear behind the camera. Changing the x value would see it moving sideways, and changing the y value would see it moving up/down. Of course, the direction in which entities appear to move is relative to the position and orientation of the camera. 
        /// </summary>
        /// <param name="vec">XYZ coordinate that entity will be positioned at </param>
        /// <param name="global">True if the position should be relative to 0, 0, 0 rather than a parent entity's position  </param>
        public void Move(Vector3 vec, bool global = false)
        {
            Xors3d.MoveEntity(_entity._handle, vec.X, vec.Y, vec.Z, global);
        }
        public void Move(float x, float y, float z, bool global = false)
        {
            Move(new Vector3(x, y, z), global);
        }


        /// <summary>
        /// Scales an entity so that it is of an absolute size. Scale values of 1, 1, 1 are the default size when creating /loading entities. Scale values of 2, 2, 2 will double the size of an entity. Scale values of 0, 0, 0 will make an entity disappear. Scale values of less than 0, 0, 0 will invert an entity and make it bigger. 
        /// </summary>
        /// <param name="vec">XYZ size of entity</param>
        /// <param name="global">Unused</param>
        public void Scale(Vector3 vec, bool global = false)
        {
            Xors3d.ScaleEntity(_entity._handle, vec.X, vec.Y, vec.Z, global);
        }
        public void Scale(float x, float y, float z, bool global = false)
        {
            Scale(new Vector3(x, y, z), global);
        }

        public void ScaleUp(float x, float y, float z, bool global = false)
        {
            ScaleUp(new Vector3(x, y, z), global);
        }
        public void ScaleUp(Vector3 vec, bool global = false)
        {
            Vector3 old = _entity.Movement.GetScale();
            old._x += vec._x; old._y += vec._y; old._z += vec._z;
            Scale(old, global);
        }
    }
}
