﻿using Xors;

namespace Xors.Physics
{
    public class CompoundShape
    {
        internal Entity _entity = null;

        /// <summary>
        /// Creates a compound shape and attaches it to an entity. 
        ///The compound acts as a container for several shapes belonging to one rigid body. A compound shape is conceptually similar to bodies held together by fixed joints; however, unlike fixed joints compound shapes are always kept perfectly rigid and incur no performance cost when simulating.
        ///Compound shape can held box, sphere, capsule, cone and cylinder subshapes. 
        ///Note:
        ///Setting mass to zero creates a static (non-dynamic) rigid body. 
        ///Warning:
        ///There is no way to change the size of the subshape after adding it to the compound shape
        /// </summary>
        /// <param name="mass"></param>
        public void AddCompoundShape(float mass)
        {
            Xors3d.EntityAddCompoundShape(_entity._handle, mass);
        }

        /// <summary>
        /// Adds a box subshape to an entity's compound shape
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="depth"></param>
        public void AddBox(float width, float height, float depth)
        {
            Xors3d.EntityCompoundAddBox(_entity._handle, width, height, depth);
        }

        /// <summary>
        /// Adds a sphere subshape to an entity's compound shape
        /// </summary>
        /// <param name="radius"></param>
        public void AddSphere(float radius)
        {
            Xors3d.EntityCompoundAddSphere(_entity._handle, radius);
        }

        /// <summary>
        /// Adds a capsule subshape to an entity's compound shape
        /// </summary>
        /// <param name="radius"></param>
        /// <param name="height"></param>
        public void AddCapsule(float radius, float height)
        {
            Xors3d.EntityCompoundAddCapsule(_entity._handle, radius, height);
        }

        /// <summary>
        /// Adds a cone subshape to an entity's compound shape
        /// </summary>
        /// <param name="radius"></param>
        /// <param name="height"></param>
        public void AddCone(float radius, float height)
        {
            Xors3d.EntityCompoundAddCone(_entity._handle, radius, height);
        }

        /// <summary>
        /// Adds a cylinder subshape to an entity's compound shape
        /// </summary>
        /// <param name="radius"></param>
        /// <param name="height"></param>
        public void AddCylinder(float radius, float height)
        {
            Xors3d.EntityCompoundAddCylinder(_entity._handle, radius, height);
        }

        /// <summary>
        /// Returns the number of compound's subshapes. 
        /// </summary>
        /// <returns></returns>
        public int CountChildren()
        {
            return Xors3d.EntityCompoundCountChildren(_entity._handle);
        }

        /// <summary>
        /// Removes a compound's subshape
        /// </summary>
        /// <param name="index"></param>
        public void RemoveChild(int index)
        {
            Xors3d.EntityCompoundRemoveChild(_entity._handle, index);
        }

        /// <summary>
        /// Sets a new position of the compound's subshape
        /// </summary>
        /// <param name="index"></param>
        /// <param name="vec"></param>
        public void ChildSetPosition(int index, Vector3 vec)
        {
            Xors3d.EntityCompoundChildSetPosition(_entity._handle, index, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Sets a new rotation of the compound's subshape
        /// </summary>
        /// <param name="index"></param>
        /// <param name="vec"></param>
        public void ChildSetRotation(int index, Vector3 vec)
        {
            Xors3d.EntityCompoundChildSetRotation(_entity._handle, index, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Returns XYZ coordinate of the local position of the compound's subshape
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Vector3 ChildGetPosition(int index)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityCompoundChildGetX(_entity._handle, index);
            v._y = Xors3d.EntityCompoundChildGetY(_entity._handle, index);
            v._z = Xors3d.EntityCompoundChildGetZ(_entity._handle, index);
            return v;
        }

        /// <summary>
        /// Returns XYZ coordinate of the local rotation of the compound's subshape
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Vector3 ChildGetRotation(int index)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityCompoundChildGetPitch(_entity._handle, index);
            v._y = Xors3d.EntityCompoundChildGetYaw(_entity._handle, index);
            v._z = Xors3d.EntityCompoundChildGetRoll(_entity._handle, index);
            return v;
        }
    }
}
