﻿namespace Xors.Physics
{
    public class JointD6 : Joint
    {
        /// <summary>
        /// Sets the limits of the specific axis of D6 or D6Spring joint
        /// </summary>
        /// <param name="axis">Joint axis. 0 - linear X axis
        ///1 - linear Y axis
        ///2 - linear Z axis
        ///3 - angular X axis
        ///4 - angular Y axis
        ///5 - angular Z axis </param>
        /// <param name="lower">Lower limit </param>
        /// <param name="upper">Upper limit </param>
        public void SetLimits(int axis, float lower, float upper)
        {
            Xors3d.JointD6SetLimits(_handle, axis, lower, upper);
        }

        /// <summary>
        /// Retrieves the current angle of D6 and D6Spring joints
        /// </summary>
        /// <returns></returns>
        public Vector3 GetAngle()
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.JointD6GetPitchAngle(_handle);
            v._y = Xors3d.JointD6GetYawAngle(_handle);
            v._z = Xors3d.JointD6GetRollAngle(_handle);
            return v;
        }

        /// <summary>
        /// Retrieves the current angle around specific axis of D6 and D6Spring joints
        /// </summary>
        /// <returns></returns>
        public Vector3 GetAngleAxis()
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.JointD6GetAngle(_handle, 0);
            v._y = Xors3d.JointD6GetAngle(_handle, 1);
            v._z = Xors3d.JointD6GetAngle(_handle, 2);
            return v;
        }

        /// <summary>
        /// Sets or gets the lower linear limits of D6 or D6Spring joint
        /// </summary>
        /// <param name="vec">Lower linear limit </param>
        public Vector3 LowerLinearLimits
        {
            set
            {
                Xors3d.JointD6SetLowerLinearLimits(_handle, value._x, value._y, value._z);
            }
            get
            {
                Vector3 v = new Vector3();
                v._x = Xors3d.JointD6GetLinearLowerX(_handle);
                v._y = Xors3d.JointD6GetLinearLowerY(_handle);
                v._z = Xors3d.JointD6GetLinearLowerZ(_handle);
                return v;
            }
        }

        /// <summary>
        /// Sets or gets the upper linear limits of D6 or D6Spring joint
        /// </summary>
        /// <param name="vec">Upper linear limit </param>
        public Vector3 UpperLinearLimits
        {
            set
            {
                Xors3d.JointD6SetUpperLinearLimits(_handle, value._x, value._y, value._z);
            }
            get
            {
                Vector3 v = new Vector3();
                v._x = Xors3d.JointD6GetLinearUpperX(_handle);
                v._y = Xors3d.JointD6GetLinearUpperY(_handle);
                v._z = Xors3d.JointD6GetLinearUpperZ(_handle);
                return v;
            }
        }

        /// <summary>
        /// Sets or gets the lower angular limits of D6 or D6Spring joint
        /// </summary>
        /// <param name="vec">Lower angular limit </param>
        public Vector3 LowerAngularLimits
        {
            set
            {
                Xors3d.JointD6SetLowerAngularLimits(_handle, value._x, value._y, value._z);
            }
            get
            {
                Vector3 v = new Vector3();
                v._x = Xors3d.JointD6GetAngularLowerX(_handle);
                v._y = Xors3d.JointD6GetAngularLowerY(_handle);
                v._z = Xors3d.JointD6GetAngularLowerZ(_handle);
                return v;
            }
        }

        /// <summary>
        /// Sets or gets the upper angular limits of D6 or D6Spring joint
        /// </summary>
        /// <param name="vec">Upper angular limit </param>
        public Vector3 UpperAngularLimits
        {
            set
            {
                Xors3d.JointD6SetUpperAngularLimits(_handle, value._x, value._y, value._z);
            }
            get
            {
                Vector3 v = new Vector3();
                v._x = Xors3d.JointD6GetAngularUpperX(_handle);
                v._y = Xors3d.JointD6GetAngularUpperY(_handle);
                v._z = Xors3d.JointD6GetAngularUpperZ(_handle);
                return v;
            }
        }

        /// <summary>
        /// Sets the linear limits of D6 or D6Spring joint
        /// </summary>
        /// <param name="upper"></param>
        /// <param name="lower"></param>
        public void SetLinearLimits(Vector3 upper, Vector3 lower)
        {
            LowerLinearLimits = lower;
            UpperLinearLimits = upper;
        }

        /// <summary>
        /// Sets the Angular limits of D6 or D6Spring joint
        /// </summary>
        /// <param name="upper"></param>
        /// <param name="lower"></param>
        public void SetAngularLimits(Vector3 upper, Vector3 lower)
        {
            LowerAngularLimits = lower;
            UpperAngularLimits = upper;
        }

        /// <summary>
        /// Enables or disables a spring on a specific DOF of D6Spring joint. If spring is enabled, the coefficients of damping and stiffness can be set. 
        /// </summary>
        /// <param name="index">DOF index (see 'Joint types')</param>
        /// <param name="enabled">Enable or disable the spring </param>
        /// <param name="damping">The coefficient of damping </param>
        /// <param name="stiffness">The coefficient of stiffness </param>
        public void SpringSetParam(int index, bool enabled, float damping = 1.0f, float stiffness = 1.0f)
        {
            Xors3d.JointD6SpringSetParam(_handle, index, enabled ? 1 : 0, damping, stiffness);
        }

    }
}
