﻿namespace Xors.Physics
{
    public class PhysicsShapes
    {
        internal Entity __entity = null;

        private CompoundShape _CompoundShape = new CompoundShape();

        internal Entity _entity
        {
            get
            {
                return __entity;
            }
            set
            {
                __entity = value;
                _CompoundShape._entity = __entity;
            }
        }

        public CompoundShape CompoundShape
        {
            get
            {
                return _CompoundShape;
            }
        }

        /// <summary>
        /// Creates a box shape and attaches it to the entity. 
        ///Set any dimension of the box shape to zero or less than zero and the shape will fit the dimensions of the entity automatically
        /// </summary>
        /// <param name="mass"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="depth"></param>
        public void AddBoxShape(float mass, float width=0.0f, float height=0.0f, float depth=0.0f)
        {
            Xors3d.EntityAddBoxShape(_entity._handle, mass, width, height, depth);
        }

        /// <summary>
        /// Creates a dummy shape and attaches it to the entity. 
        ///A dummy shape has no mass and collision body.
        /// </summary>
        public void AddDummyShape()
        {
            Xors3d.EntityAddDummyShape(_entity._handle);
        }

        /// <summary>
        /// Creates a sphere shape and attaches it to the entity. 
        ///Set the radius of the sphere shape to zero or less than zero and the shape will fit the dimensions of the entity automatically
        /// </summary>
        /// <param name="mass"></param>
        /// <param name="radius"></param>
        public void AddSphereShape(float mass, float radius = 0.0f)
        {
            Xors3d.EntityAddSphereShape(_entity._handle, mass, radius);
        }

        /// <summary>
        /// Creates a capsule shape and attaches it to the entity. 
        ///Set any dimension of the capsule shape to zero or less than zero and the shape will fit the dimensions of the entity automatically
        /// </summary>
        /// <param name="mass"></param>
        /// <param name="radius"></param>
        /// <param name="height"></param>
        public void AddCapsuleShape(float mass, float radius = 0.0f, float height = 0.0f)
        {
            Xors3d.EntityAddCapsuleShape(_entity._handle, mass, radius, height);
        }

        /// <summary>
        /// Creates a cone shape and attaches it to the entity. 
        ///Set any dimension of the cone shape to zero or less than zero and the shape will fit the dimensions of the entity automatically
        /// </summary>
        /// <param name="mass"></param>
        /// <param name="radius"></param>
        /// <param name="height"></param>
        public void AddConeShape(float mass, float radius = 0.0f, float height = 0.0f)
        {
            Xors3d.EntityAddConeShape(_entity._handle, mass, radius, height);
        }

        /// <summary>
        /// Creates a cylinder shape and attaches it to the entity. 
        ///Set any dimension of the cylinder shape to zero or less than zero and the shape will fit the dimensions of the entity automatically. 
        /// </summary>
        /// <param name="mass"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="depth"></param>
        public void AddCylinderShape(float mass, float width = 0.0f, float height = 0.0f, float depth = 0.0f)
        {
            Xors3d.EntityAddCylinderShape(_entity._handle, mass, width, height, depth);
        }

        /// <summary>
        /// Creates a trimesh (triangle mesh) shape and attaches it to the entity. 
        ///Created trimesh shape is static which means that moving of vertices won't affect the shape
        /// </summary>
        public void AddTriMeshShape()
        {
            Xors3d.EntityAddTriMeshShape(_entity._handle);
        }

        /// <summary>
        /// Creates a trimesh (triangle mesh) shape using information about vertices and indices of the proxy entity and attaches it to the main entity. 
        ///Created trimesh shape is static which means that moving of vertices won't affect the shape.
        ///Usually proxy entity is a lowpoly version of the main entity. Proxy entity may be freed after using in this function
        /// </summary>
        /// <param name="proxy"></param>
        public void AddTriMeshShapeProxy(Entity proxy)
        {
            Xors3d.EntityAddTriMeshShapeProxy(_entity._handle, proxy._handle);
        }

        /// <summary>
        /// Creates a convex shape and attaches it to the entity. 
        ///The convex shape works much faster than the concave one. That's why the use of the convex shape is more preferable if it fits your needs
        /// </summary>
        /// <param name="mass"></param>
        public void AddConvexShape(float mass)
        {
            Xors3d.EntityAddConvexShape(_entity._handle, mass);
        }

        /// <summary>
        /// Creates a convex shape using information about vertices and indices of the proxy entity and attaches it to the main entity. 
        ///The convex shape works much faster than the concave one. That's why the use of the convex shape is more preferable if it fits your needs. 
        ///Note:
        ///Setting mass to zero creates a static (non-dynamic) rigid body. Usually proxy entity is a lowpoly version of the main entity. Proxy entity may be freed after using in this function
        /// </summary>
        /// <param name="proxy"></param>
        /// <param name="mass"></param>
        public void AddConvexShapeProxy(Entity proxy, float mass)
        {
            Xors3d.EntityAddConvexShapeProxy(_entity._handle, proxy._handle, mass);
        }

        /// <summary>
        /// Creates a concave shape and attaches it to the entity. 
        ///Unlike a convex shape a concave shape can have dents and holes. For instance, a torus will have a hole if it was created as a concave shape. 
        ///Note:
        ///Setting mass to zero creates a static (non-dynamic) rigid body. 
        /// </summary>
        /// <param name="mass"></param>
        public void AddConcaveShape(float mass)
        {
            Xors3d.EntityAddConcaveShape(_entity._handle, mass);
        }

        /// <summary>
        /// Creates a concave shape using information about vertices and indices of the proxy entity and attaches it to the main entity. 
        ///Unlike a convex shape a concave shape can have dents and holes. For instance, a torus will have a hole if it was created as a concave shape. 
        ///Note:
        ///Setting mass to zero creates a static (non-dynamic) rigid body. Usually proxy entity is a lowpoly version of the main entity. Proxy entity may be freed after using in this function. 
        /// </summary>
        /// <param name="proxy"></param>
        /// <param name="mass"></param>
        public void AddConcaveShapeProxy(Entity proxy, float mass)
        {
            Xors3d.EntityAddConcaveShapeProxy(_entity._handle, proxy._handle, mass);
        }

        /// <summary>
        /// Creates a static shape for a terrain. Can be used only with the terrains
        /// </summary>
        public void AddTerrainShape()
        {
            Xors3d.EntityAddTerrainShape(_entity._handle);
        }

        /// <summary>
        /// Attaches a previously detached body.
        /// </summary>
        /// <param name="body"></param>
        public void AttachBody(Body body)
        {
            Xors3d.EntityAttachBody(_entity._handle, body._handle);
        }

        /// <summary>
        /// Detaches a body from an entity
        /// </summary>
        /// <returns></returns>
        public Body DetachBody()
        {
            Body body = new Body();
            body._handle = Xors3d.EntityDetachBody(_entity._handle);
            return body._handle > 0 ? body : null;
        }

    }
}
