﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class Shadows
    {

        static internal bool AutoShadows = false;

        /// <summary>
        /// Sets the automatic adjustment of shadows
        /// </summary>
        public static void SetAutoShadows()
        {
            AutoShadows = true;
            Graphics.CreateDSS(1024, 1024);
            SetShadowsBlur(ShadowBlurLevel.SHADOWS_BLUR_13);
            InitShadows(1024, 0, 512);
            SetShadowParams(4, 0.85f, true, 300);
        }


        /// <summary>
        /// Sets the level of Gaussian blur for the shadows
        /// </summary>
        /// <param name="blur"></param>
        public static void SetShadowsBlur(ShadowBlurLevel blur)
        {
            Xors3d.SetShadowsBlur((int)blur);
        }

        /// <summary>
        /// Sets a path to the shadow shader (e.g. "shaders\mySmartShadows.hlsl"). 
        /// Specified shader must contain code for rendring shadow maps (writing depth textures), final scene rendering with shadows, and mixing the result from all light sources. If no shader is specified in this function - default built-in shader is used
        /// </summary>
        /// <param name="path"></param>
        public static void SetShadowShader(string path)
        {
            Xors3d.SetShadowShader(path);
        }

        /// <summary>
        /// Initializes the shadow system and returns the handle of the shadow shader. 
        /// You should call this function to use shadows. You can specify the size of the shadow maps for all types of lights separately. If you set 0 the shadow map will not be created for this type of light, you can't cast shadows from this light, but you will save some video memory. 
        /// </summary>
        /// <param name="dirSize"></param>
        /// <param name="spotSize"></param>
        /// <param name="pointSize"></param>
        public static Effect InitShadows(int dirSize, int spotSize, int pointSize)
        {
            Effect e = new Effect();
            e._handle = Xors3d.InitShadows(dirSize, spotSize, pointSize);
            return e._handle > 0 ? e : null;
        }

        /// <summary>
        /// Sets parameters of the shadow system. 
        /// You can change number of PSSM split planes (PSSM technique splits visible scene in some parts, each part will be rendered separately to its own shadow map. It increases quality of shadows near camera, but may cause loss of quality in large distance from camera), and split lambda. We set optimal values, but you may change them for better quality or better perfomance. Also you can change projection type for shadow map rendering, and set the distance of light source from the camera view point (larger distance causes larger part of scene which casts shadows, but quality of the shadows is poorer). 
        /// </summary>
        /// <param name="splitPlanes">Number of split planes for PSSM rendering. The number of split planes is equal to the number of shadow maps for the directional light</param>
        /// <param name="splitLambda">The ratio of the scene splitting in range (0.0; 1.0). </param>
        /// <param name="useOrtho">True to use orthographical projection, false - for perspective projection</param>
        /// <param name="lightDist">Distance from light source to the view point</param>
        public static void SetShadowParams(int splitPlanes = 4, float splitLambda = 0.95f, bool useOrtho = true, float lightDist = 300.0f)
        {
            Xors3d.SetShadowParams(splitPlanes, splitLambda, useOrtho, lightDist);
        }

        /// <summary>
        /// Sets the minimum priority of the shadow casting. 
        /// All light sources have the shadow casting priority. If priority is greater than zero - light source will cast shadows. But you can set the global priority and shadow casting will be disabled for all light sources, which priority is less than the global one. You can use it for changing shadow quality in your game (enable more light for powerful machines, and disable it for old hardware) using one command.
        /// </summary>
        /// <param name="priority"></param>
        public static void SetShadowPriority(int priority)
        {
            Xors3d.ShadowPriority(priority);
        }
    }
}
