﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Sound
    {
        internal int _handle = 0;

        public bool equals(Sound value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Frees up a sound
        /// </summary>
        public void Free()
        {
            Xors3d.FreeSound(_handle);
        }

        /// <summary>
        /// Sets up play back a sound file in an endless loop (like for background music) 
        /// This command doesn't actually play the sound loop, just sets it up for looping. You still need to execute the xPlaySound() command to hear the sound 
        /// </summary>
        public void SetLoop()
        {
            Xors3d.LoopSound(_handle);
        }

        /// <summary>
        /// Alters the pitch of a sound. 
        /// By changing the pitch, you can often reuse sounds for different uses or to simulate a 'counting up/down' sound. To make the sound 'higher pitched', increase the hertz. Conversely, decreasing the hertz will 'lower' the pitch. Note: this is in relation to the original hertz frequency of the sound.
        /// </summary>
        /// <param name="pitch">Valid playback hertz speed (up to 44000 hertz)</param>
        public void SetPitch(int pitch)
        {
            Xors3d.SoundPitch(_handle, pitch);
        }

        /// <summary>
        /// Alters the playback volume of sound effect. 
        /// This command uses a floating point number from 0.0 to 1.0 to control the volume level 
        /// </summary>
        /// <param name="volume"></param>
        public void SetVolume(float volume)
        {
            Xors3d.SoundVolume(_handle, volume);
        }

        /// <summary>
        /// Pans sound effect between the left and right speakers
        /// </summary>
        /// <param name="pan">Floating point number from -1.0 (left) to 0.0 (center) to 1.0 (right)</param>
        public void SetPan(float pan)
        {
            Xors3d.SoundPan(_handle, pan);
        }

        /// <summary>
        /// Plays a sound previously loaded using the xLoadSound() command
        /// </summary>
        /// <returns></returns>
        public SoundChannel Play()
        {
            SoundChannel sc = new SoundChannel();
            sc._handle = Xors3d.PlaySound(_handle);//hz -1 or 0 handle
            return sc;
        }

        /// <summary>
        /// Emits a sound attached to the specified entity and returns a sound channel. 
        /// The sound must have been loaded using xLoad3DSound() for 3D effects
        /// </summary>
        /// <param name="entity"></param>
        public SoundChannel Emit(Entity entity)
        {
            SoundChannel sc = new SoundChannel();
            sc._handle = Xors3d.EmitSound(_handle, entity._handle);
            return sc;
        }

        /// <summary>
        /// Creates a listener entity and returns its handle. 
        /// Currently, only a single listener is supported
        /// </summary>
        /// <param name="parent"></param>
        /// <param name="roFactor"></param>
        /// <param name="doplerFactor"></param>
        /// <param name="distFactor"></param>
        /// <returns></returns>
        public Entity CreateListener(Entity parent = null, float roFactor = 1.0f, float doplerFactor = 1.0f, float distFactor = 1.0f)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CreateListener(parent != null ? parent._handle : 0, roFactor, doplerFactor, distFactor);
            return ent._handle > 0 ? ent : null;
        }
    }
}
