﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Sprite
    {
        internal int _handle = 0;

        public bool equals(Sprite value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Sets image rendering color
        /// </summary>
        public Color Color
        {
            set
            {
                Xors3d.ImageColor(_handle, value._red, value._green, value._blue);
            }
        }

        /// <summary>
        /// Sets image alpha value
        /// </summary>
        public float Alpha
        {
            set
            {
                Xors3d.ImageAlpha(_handle, value);
            }
        }

        /// <summary>
        /// Returns specified image buffer handle. 
        /// You may use it for directly drawing into image 
        /// </summary>
        /// <param name="frame"></param>
        /// <returns></returns>
        public ImageBuffer GetImageBuffer(int frame = 0)
        {
            ImageBuffer ib = new ImageBuffer();
            ib._handle = Xors3d.ImageBuffer(_handle, frame);
            return ib._handle > 0 ? ib : null;
        }

        /// <summary>
        /// Grabs a portion of the current drawing buffer and stick it into an image
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="frame">Frame to insert the grabbed image into </param>
        public void GrabImage(int x, int y, int frame = 0)
        {
            Xors3d.GrabImage(_handle, x, y, frame);
        }

        /// <summary>
        /// Frees up an image
        /// </summary>
        public void Free()
        {
            Xors3d.FreeImage(_handle);
        }

        /// <summary>
        /// Saves an image or one of its frames to hard drive
        /// </summary>
        /// <param name="path"></param>
        /// <param name="?"></param>
        public void SaveImage(string path, int frame = 0)
        {
            Xors3d.SaveImage(_handle, path, frame);
        }

        /// <summary>
        /// Draws a previously loaded image. 
        /// This command draws both single image graphics (loaded with the xLoadImage() command) as well as animated images (loaded with the xLoadAnimImage() command). You specify where on the screen you wish the image to appear. You can actually 'draw' off the screen as well by using negative values or positive values that are not visible 'on the screen'. Finally, if you are using an animated image (loaded with the xLoadAnimImage()), you can specify which frame of the imagestrip is displayed with the xDrawImage() command. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="frame"></param>
        public void Draw(float x, float y, int frame = 0)
        {
            Xors3d.DrawImage(_handle, x, y, frame);
        }

        /// <summary>
        /// Draws a rectangular portion of an image to the designated location on the screen. 
        /// The transparent/masked portions of the original image will be drawn transparent, just as you normally would draw an image
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="rectx"></param>
        /// <param name="recty"></param>
        /// <param name="rectWidth"></param>
        /// <param name="rectHeight"></param>
        /// <param name="frame"></param>
        public void DrawImageRect(float x, float y, float rectx, float recty, float rectWidth, float rectHeight, int frame = 0)
        {
            Xors3d.DrawImageRect(_handle, x, y, rectx, recty, rectWidth, rectHeight, frame);
        }

        /// <summary>
        /// Resizes an image to a new size using a floating point percentage. 
        /// Using a negative value perform image flipping. You may use this in realtime
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void Scale(float x, float y)
        {
            Xors3d.ScaleImage(_handle, x, y);
        }

        /// <summary>
        /// Resizes an image to a new size using a values in pixels
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        public void Resize(float width, float height)
        {
            Xors3d.ResizeImage(_handle, width, height);
        }

        /// <summary>
        /// Rotates images. 
        /// The purpose of this command is to rotate an image a specified number of degrees. You my use it in realtime
        /// </summary>
        /// <param name="angle"></param>
        public float RotateAngle
        {
            set
            {
                Xors3d.RotateImage(_handle, value);
            }
            get
            {
                return Xors3d.ImageAngle(_handle);
            }
        }

        public int GetWidth()
        {
            return Xors3d.ImageWidth(_handle);
        }

        public int GetHeight()
        {
            return Xors3d.ImageHeight(_handle);
        }

        /// <summary>
        /// Checks if images are collided. 
        /// This is the command to get pixel-perfect collisions between images. It will not consider transparent pixels during the collision check (basically, only the 'meat' of the image will invoke a collision). This makes it perfect for most situations where you have odd-shaped graphics to text against. The xImagesOverlap() command is mesh faster, however, but can only determine if any of the two images have overlapped (this includes transparent pixels). This method works if you have graphics that completely fill their container and/or you don't plan on needing pinpoint accuracy. 
        /// </summary>
        /// <param name="x1"></param>
        /// <param name="y1"></param>
        /// <param name="frame1"></param>
        /// <param name="image2"></param>
        /// <param name="x2"></param>
        /// <param name="y2"></param>
        /// <param name="frame2"></param>
        /// <returns></returns>
        public bool CollideWithSprite(int x1, int y1, int frame1, Sprite image2, int x2, int y2, int frame2)
        {
            return Xors3d.ImagesCollide(_handle, x1, x2, frame1, image2._handle, x2, y2, frame2)==1;
        }

        /// <summary>
        /// Checks if image is collided with a rectangle on the screen. 
        /// There are many times when you need to see if an image has collided with (or is touching) a specific rectangular area of the screen. This command performs pixel perfect accurate collision detection between the image of your choice and a specified rectangle on the screen. Howevever, should your program just need to detect a graphic (like a mouse pointer) over at a particular location/region of the screen (often called a 'hot spot'), this command works great. As with any collision, you will need to know the PRECISE location of the graphic you wish to test collision with, as well as the x, y, width, and height of the screen area (rect) you wish to test. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="frame"></param>
        /// <param name="rectx"></param>
        /// <param name="recty"></param>
        /// <param name="rectWidth"></param>
        /// <param name="rectHeight"></param>
        /// <returns></returns>
        public bool CollideWithRect(int x, int y, int frame, int rectx, int recty, int rectWidth, int rectHeight)
        {
            return Xors3d.ImageRectCollide(_handle, x, y, frame, rectx, recty, rectWidth, rectHeight)==1;
        }

        /// <summary>
        /// Checks if image is overlapped with a rectangle on the screen
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="rectx"></param>
        /// <param name="recty"></param>
        /// <param name="rectWidth"></param>
        /// <param name="rectHeight"></param>
        /// <returns></returns>
        public bool RectOverlap(float x, float y, float rectx, float recty, float rectWidth, float rectHeight)
        {
            return Xors3d.ImageRectOverlap(_handle, x, y, rectx, recty, rectWidth, rectHeight)==1;
        }

        /// <summary>
        /// Sets image drawing handle position. 
        /// When an image is loaded with xLoadImage(), the image handle (the location within the image where the image is 'drawn from') is always defaulted to the top left corner (coordinates 0, 0). This means if you draw an image that is 50x50 pixels at screen location 200, 200, the image will begin to be drawn at 200, 200 and extend to 250, 250. This command moves the image handle from the 0, 0 coordinate of the image to the specified x and y location in the image. You can retrieve an image's current location handle using the xImageXHandle() and xImageYHandle(). Finally, you can make all images automatically load with the image handle set to middle using the xAutoMidHandle() command. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void SetImageHandle(float x, float y)
        {
            Xors3d.HandleImage(_handle, x, y);
        }

        /// <summary>
        /// Returns x location of an image's drawing handle
        /// </summary>
        /// <returns></returns>
        public float GetImageHandleX()
        {
            return Xors3d.ImageXHandle(_handle);
        }

        /// <summary>
        /// Returns y location of an image's drawing handle
        /// </summary>
        /// <returns></returns>
        public float GetImageHandleY()
        {
            return Xors3d.ImageYHandle(_handle);
        }

        /// <summary>
        /// Centers image drawing handle. 
        /// When an image is loaded with xLoadImage(), the image handle (the location within the image where the image is 'drawn from') is always defaulted to the top left corner (coordinates 0, 0). This means if you draw an image that is 50x50 pixels at screen location 200, 200, the image will begin to be drawn at 200, 200 and extend to 250, 250. This command moves the image handle from the 0, 0 coordinate of the image to the exact middle of the image. Therefore, in the same scenario above, if you were to draw a 50x50 pixel image at screen location 200, 200 with its image handle set to Mid with this command, the image would start drawing at 175, 175 and extend to 225, 225. You can manual set the location of the image's handle using the xHandleImage() command. You can retrieve an image's handle using the xImageXHandle() and xImageYHandle(). Finally, you can make all images automatically load with the image handle set to middle using the xAutoMidHandle() command. 
        /// </summary>
        public void MidHandle()
        {
            Xors3d.MidHandle(_handle);
        }

        /// <summary>
        /// Enables or disables auto appling xMidhandle() for all loading images. 
        /// When an image is loaded with xLoadImage(), the image handle (the location within the image where the image is 'drawn from') is always defaulted to the top left corner (coordinates 0, 0). This means if you draw an image that is 50x50 pixels at screen location (200, 200), the image will begin to be drawn at (200, 200) and extend to (250, 250). The xMidHandle() command moves the image's handle to the middle of the image. See this command for more information about the image's handle. This command eliminates the need for the xMidHandle() command by making ALL subsequently loaded images default to having their image handles set to mid. 
        /// </summary>
        /// <param name="value"></param>
        public void SetAutoMidHandle(bool value)
        {
            Xors3d.AutoMidHandle(value ? 1 : 0);
        }

        /// <summary>
        /// Tiles image on the screen
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="frame"></param>
        public void Tile(float x, float y, int frame = 0)
        {
            Xors3d.TileImage(_handle, x, y, frame);
        }

        /// <summary>
        /// Checks if images are overlapped. 
        /// This is a very fast, simple collision type command that will allow you to determine whether or not two images have overlapped each other. This does not take into account any transparent pixels (see xImagesCollide()). In many cases, you might be able to get away with using this more crude, yet quite fast method of collision detection. For games where your graphics are very squared off and pixel-perfect accuracy isn't a must, you can employ this command to do quick and dirty overlap checking. 
        /// </summary>
        /// <param name="x1"></param>
        /// <param name="y1"></param>
        /// <param name="image2"></param>
        /// <param name="x2"></param>
        /// <param name="y2"></param>
        /// <returns></returns>
        public bool OverlapWithImage(float x1, float y1, Sprite image2, float x2, float y2)
        {
            return Xors3d.ImagesOverlap(_handle, x1, y1, image2._handle, x2, y1)==1;
        }

        /// <summary>
        /// Sets image transparent color
        /// </summary>
        /// <param name="color"></param>
        public void MaskImage(Color color)
        {
            Xors3d.MaskImage(_handle, color._red, color._green, color._blue);
        }

        /// <summary>
        /// Creates image's copy. 
        /// </summary>
        /// <returns></returns>
        public Sprite Copy()
        {
            Sprite s = new Sprite();
            s._handle = Xors3d.CopyImage(_handle);
            return s._handle>0 ? s : null;
        }

        /// <summary>
        /// Draws a previously loaded image without transparent blocks
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="frame"></param>
        public void DrawBlock(float x, float y, int frame = 0)
        {
            Xors3d.DrawBlock(_handle, x, y, frame);
        }

        /// <summary>
        /// Draws a rectangular portion of an image to the designated location on the screen without transparent blocks
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="rectx"></param>
        /// <param name="recty"></param>
        /// <param name="rectWidth"></param>
        /// <param name="rectHeight"></param>
        /// <param name="frame"></param>
        public void DrawBlockRect(float x, float y, float rectx, float recty, float rectWidth, float rectHeight, int frame = 0)
        {
            Xors3d.DrawBlockRect(_handle, x, y, rectx, recty, rectWidth, rectHeight, frame);
        }

        /// <summary>
        /// Returns the width of the given image in pixels (images scale is taken into account)
        /// </summary>
        /// <returns></returns>
        public int GetImageActualWidth()
        {
            return Xors3d.ImageActualWidth(_handle);
        }

        /// <summary>
        /// Returns the height of the given image in pixels (images scale is taken into account).
        /// </summary>
        /// <returns></returns>
        public int GetImageActualHeight()
        {
            return Xors3d.ImageActualHeight(_handle);
        }


    }
}
