﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class State
    {
        internal Entity _entity = null;

        /// <summary>
        /// Returns a parent of an entity. 
        /// </summary>
        /// <returns>If parent handle is 0, returned null</returns>
        public Entity GetParent()
        {
            int p = Xors3d.GetParent(_entity._handle);
            if (p <= 0) return null;
            Entity ent = new Entity();
            ent._handle = p;
            return ent;
        }

        /// <summary>
        /// Returns the distance between this and destination entities. 
        /// </summary>
        /// <param name="destination">Destination entity </param>
        /// <returns></returns>
        public float EntityDistance(Entity destination)
        {
            return Xors3d.EntityDistance(_entity._handle, destination._handle);
        }

        /// <summary>
        /// Returns the value of an element from within an entity's world matrix. 
        /// </summary>
        /// <param name="row">Matrix row index</param>
        /// <param name="col">Matrix cell index</param>
        /// <returns></returns>
        public float GetMatElement(int row, int col)
        {
            return Xors3d.GetMatElement(_entity._handle, row, col);
        }

        /// <summary>
        /// Returns a string containing the class of the given entity. 
        /// </summary>
        /// <returns></returns>
        public string EntityClass()
        {
            return Xors3d.EntityClass(_entity._handle);
        }

        /// <summary>
        /// Returns an entity's brush. Remember, xGetSurfaceBrush() actually creates a new brush so don't forget to free it afterwards using xFreeBrush() to prevent memory leaks. Once you have got the brush handle from a surface, you can use xGetBrushTexture() and xTextureName() to get the details of what texture(s) are applied to the brush. 
        /// </summary>
        /// <returns></returns>
        public Brush GetBrush()
        {
            Brush brush = new Brush();
            brush._handle = Xors3d.GetEntityBrush(_entity._handle);
            return brush;
        }      

        /// <summary>
        /// Returns true if entities can see each other. 
        /// </summary>
        /// <param name="destination">Destination entity</param>
        /// <returns></returns>
        public bool EntityVisible(Entity destination)
        {
            return Xors3d.EntityVisible(_entity._handle,destination._handle)==1?true:false;
        }


        

        /// <summary>
        /// Returns the name of an entity. An entity's name may be set in a modelling program, or manually set
        /// </summary>
        /// <returns></returns>
        public string Name
        {
            get
            {
                return Xors3d.EntityName(_entity._handle);
            }
            set
            {
                Xors3d.NameEntity(_entity._handle, value);
            }
        }

        /// <summary>
        /// Returns the number of children of an entity.
        /// </summary>
        /// <returns></returns>
        public int CountChildren()
        {
            return Xors3d.CountChildren(_entity._handle);
        }

        /// <summary>
        /// Returns a child of an entity. If child not exists then return null
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Entity GetChild(int index)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.GetChild(_entity._handle, index);
            if (ent._handle > 0) return ent;
            else return null;
        }

        /// <summary>
        /// Returns the first child of the specified entity with matching name
        /// </summary>
        /// <param name="name">Child name to find within entity </param>
        /// <returns></returns>
        public Entity FindChild(string name)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.FindChild(_entity._handle, name);
            if (ent._handle > 0) return ent;
            else return null;
        }

        /// <summary>
        /// Get or set a entity's world matrix. In handle
        /// </summary>
        /// <returns></returns>
        public D3DXMATRIX EntityMatrix
        {
            get
            {
                D3DXMATRIX matrix = new D3DXMATRIX();
                matrix._handle = Xors3d.GetEntityMatrix(_entity._handle);
                return matrix;
            }
            set
            {
                Xors3d.SetEntityMatrix(_entity._handle, value._handle);
            }
        }

        /// <summary>
        /// get or set entity alpha
        /// </summary>
        public float Alpha
        {
            get
            {
                return Xors3d.GetEntityAlpha(_entity._handle);
            }
            set
            {
                Xors3d.EntityAlpha(_entity._handle, value);
            }
        }

        /// <summary>
        /// get or set entity's alpha testing reference
        /// </summary>
        public int AlphaRef
        {
            get
            {
                return Xors3d.GetAlphaRef(_entity._handle);
            }
            set
            {
                Xors3d.EntityAlphaRef(_entity._handle, value);
            }
        }

        /// <summary>
        /// get or set entity's alpha testing function
        /// </summary>
        public int AlphaFunc
        {
            get
            {
                return Xors3d.GetAlphaFunc(_entity._handle);
            }
            set
            {
                Xors3d.EntityAlphaFunc(_entity._handle, value);
            }
        }

        /// <summary>
        /// Get or set XYZ of color x = red, y = green, z = blue
        /// </summary>
        /// <returns></returns>
        public Color ColorRGB
        {
            get
            {
                Color v = new Color();
                v._red = Xors3d.EntityRed(_entity._handle);
                v._green = Xors3d.EntityGreen(_entity._handle);
                v._blue = Xors3d.EntityBlue(_entity._handle);
                return v;
            }
            set
            {
                Xors3d.EntityColor(_entity._handle, (int)value._red, (int)value._green, (int)value._blue);
            }
        }

        /// <summary>
        /// get or set entity shininess
        /// </summary>
        public float Shininess
        {
            get
            {
                return Xors3d.GetEntityShininess(_entity._handle);
            }
            set
            {
                Xors3d.EntityShininess(_entity._handle, value);
            }
        }

        /// <summary>
        /// get or set entity blending mode
        /// </summary>
        public Blend Blend
        {
            get
            {
                return (Blend)Xors3d.GetEntityBlend(_entity._handle);
            }
            set
            {
                Xors3d.EntityBlend(_entity._handle, (int)value);
            }
        }

        /// <summary>
        /// get or set entity fx mode
        /// </summary>
        public FX FX
        {
            get
            {
                return (FX)Xors3d.GetEntityFX(_entity._handle);
            }
            set
            {
                Xors3d.EntityFX(_entity._handle, (int)value);
            }
        }

        /// <summary>
        /// Returns true if entity was hidden by the xHideEntity() function and false otherwise. 
        /// </summary>
        /// <returns></returns>
        public bool EntityHidden()
        {
            return Xors3d.EntityHidden(_entity._handle)==1?true:false;
        }

        /// <summary>
        /// Returns true if the specified meshes bounding boxes are currently intersecting. 
        /// </summary>
        /// <param name="destination">Second entity</param>
        /// <returns></returns>
        public bool EntitiesBBIntersect(Entity destination)
        {
            return Xors3d.EntitiesBBIntersect(_entity._handle, destination._handle)==1?true:false;
        }

        /// <summary>
        /// Returns the first entity between x, y, z to dx, dy, dz.
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="z"></param>
        /// <param name="dx"></param>
        /// <param name="dy"></param>
        /// <param name="dz"></param>
        /// <param name="distance"></param>
        /// <returns></returns>
        public Entity LinePick(float x,  float y,  float z,  float dx,  float dy,  float dz,  float distance = 0.0f)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.LinePick(x, y, z, dx, dy, dz, distance);
            if (ent._handle > 0) return ent;
            else return null;
        }

        /// <summary>
        /// Returns the nearest entity 'ahead' of the specified entity. An entity must have a non-zero xEntityPickMode() to be pickable
        /// </summary>
        /// <param name="range">Range of pick area around entity</param>
        /// <returns></returns>
        public Entity EntityPick(float range = 0)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.EntityPick(_entity._handle, range);
            if (ent._handle > 0) return ent;
            else return null;
        }

        /// <summary>
        /// Returns the pitch angle, that first entity should be rotated by in order to face second. This command can be used to be point one entity at another, rotating on the x axis only 
        /// </summary>
        /// <param name="second"></param>
        /// <returns></returns>
        public float DeltaPitch(Entity second)
        {
            return Xors3d.DeltaPitch(_entity._handle, second._handle);
        }

        /// <summary>
        /// Returns the yaw angle, that first entity should be rotated by in order to face second. This command can be used to be point one entity at another, rotating on the y axis only 
        /// </summary>
        /// <param name="second"></param>
        /// <returns></returns>
        public float DeltaYaw(Entity second)
        {
            return Xors3d.DeltaYaw(_entity._handle, second._handle);
        }

        public void SetFrustumSphere(Vector3 vec, float radii)
        {
            Xors3d.SetFrustumSphere(_entity._handle, vec._x, vec._y, vec._z, radii);
        }

        public void CalculateFrustumVolume()
        {
            Xors3d.CalculateFrustumVolume(_entity._handle);
        }

        
    }
}
