﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Surface
    {
        internal int _handle = 0;
        private ShaderEffectSurface _ShaderEffect = new ShaderEffectSurface();

        public bool equals(Surface value)
        {
            return value._handle == _handle;
        }

        public ShaderEffectSurface ShaderEffect
        {
            get
            {
                return _ShaderEffect;
            }
        }

        public Surface()
        {
            _ShaderEffect._surface = this;
        }

        /// <summary>
        /// Sets surface's bounding sphere for frustum culling
        /// </summary>
        /// <param name="vec"></param>
        /// <param name="radii">Radius of bounding sphere </param>
        public void SetFrustumSphere(Vector3 vec, float radii)
        {
            Xors3d.SetSurfaceFrustumSphere(_handle, vec._x, vec._y, vec._z, radii);
        }

        /// <summary>
        /// Returns a brush with the same properties as is applied to the specified mesh surface. 
        /// If this command does not appear to be returning a valid brush, try using xGetEntityBrush() instead. Remember, xGetSurfaceBrush() actually creates a new brush so don't forget to free it afterwards using xFreeBrush() to prevent memory leaks. Once you have got the brush handle from a surface, you can use xGetBrushTexture() and xTextureName() to get the details of what texture(s) are applied to the brush. 
        /// </summary>
        /// <returns></returns>
        public Brush GetBrush()
        {
            Brush b = new Brush();
            b._handle = Xors3d.GetSurfaceBrush(_handle);
            return b._handle > 0 ? b : null;
        }

        /// <summary>
        /// Adds a vertex to the specified surface and returns the vertices index number, starting from 0. 
        /// 'x', 'y', 'z' are the geometric coordinates of the vertex, and 'u', 'v', 'w' are texture mapping coordinates. A vertex is a point in 3D space which is used to connect edges of a triangle together. Without any vertices, you can't have any triangles. At least three vertices are needed to create one triangle; one for each corner. The optional 'u', 'v' and 'w' parameters allow you to specify texture coordinates for a vertex, which will determine how any triangle created using those vertices will be texture mapped. The 'u', 'v' and 'w' parameters specified will take effect on both texture coordinate sets (0 and 1). This works on the following basis:
        /// The top left of an image has the uv coordinates 0.0, 0.0.
        /// The top right has coordinates 1.0, 0.0
        /// The bottom right is 1.0, 1.0
        /// The bottom left 0.0, 1.0
        /// Thus, uv coordinates for a vertex correspond to a point in the image. For example, coordinates 0.9, 0.1 would be near the upper right corner of the image. So now imagine you have a normal equilateral triangle. By assigning the bottom left vertex a uv coordinate of 0.0, 0.0, the bottom right a coordinate of 1.0, 0.0 and the top centre 0.5, 1.0, this will texture map the triangle with an image that fits it. When adding a vertex its default color is 255, 255, 255, 255. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="z"></param>
        /// <param name="u"></param>
        /// <param name="v"></param>
        /// <param name="w"></param>
        /// <returns></returns>
        public int AddVertex(float x, float y, float z, float u = 0.0f, float v = 0.0f, float w = 0.0f)
        {
            return Xors3d.AddVertex(_handle, x, y, z, u, v, w);
        }

        /// <summary>
        /// Adds a triangle to a surface and returns the triangle's index number, starting from 0. 
        /// The v0, v1 and v2 parameters are the index numbers of the vertices created using xAddVertex(). Depending on how the vertices are arranged, then the triangle will only be visible from a certain side. Imagine that a triangle's vertex points are like dot-to-dot pattern, each numbered v0, v1, v2. If these dots, starting from v0, through to V2, form a clockwise pattern relative to the viewer, then the triangle will be visible. If these dots form an anti-clockwise pattern relative to the viewer, then the triangle will not be visible. The reason for having one-sided triangles is that it reduces the amount of triangles that need to be rendered when one side faces the side of an object which won't be seen (such as the inside of a snooker ball). However, if you wish for a triangle to be two-sided, then you can either create two triangles, using the same set of vertex numbers for both but assigning them in opposite orders, or you can use xCopyEntity() and xFlipMesh() together. 
        /// </summary>
        /// <param name="v0"></param>
        /// <param name="v1"></param>
        /// <param name="v2"></param>
        /// <returns></returns>
        public int AddTriangle(int v0, int v1, int v2)
        {
            return  Xors3d.AddTriangle(_handle, v0, v1, v2);
        }

        /// <summary>
        /// Sets the geometric coordinates of an existing vertex. 
        /// This is the command used to perform what is commonly referred to as 'dynamic mesh deformation'. It will reposition a vertex so that all the triangle edges connected to it, will move also. This will give the effect of parts of the mesh suddenly deforming
        /// </summary>
        /// <param name="vertex"></param>
        /// <param name="coords"></param>
        public void VertexCoords(int vertex, Vector3 coords)
        {
            Xors3d.VertexCoords(_handle, vertex, coords._x, coords._y, coords._z);
        }

        public void VertexTangent(int vertex, Vector3 coords)
        {
            Xors3d.VertexTangent(_handle, vertex, coords._x, coords._y, coords._z);
        }

        public void VertexNormal(int vertex, Vector3 coords)
        {
            Xors3d.VertexNormal(_handle, vertex, coords._x, coords._y, coords._z);
        }

        public void VertexBinormal(int vertex, Vector3 coords)
        {
            Xors3d.VertexBinormal(_handle, vertex, coords._x, coords._y, coords._z);
        }

        public void VertexColor(int vertex, Color color)
        {
            Xors3d.VertexColor(_handle, vertex, color._red, color._green, color._blue, color._alpha);
        }

        /// <summary>
        /// Sets the texture coordinates of an existing vertex
        /// </summary>
        /// <param name="vertex"></param>
        /// <param name="u"></param>
        /// <param name="v"></param>
        /// <param name="w"></param>
        /// <param name="textureSet"></param>
        public void VertexTexCoords(int vertex, float u, float v, float w = 1.0f, int textureSet = 0)
        {
            Xors3d.VertexTexCoords(_handle, vertex, u, v, w, textureSet);
        }

        /// <summary>
        /// Returns the number of vertices in a surface
        /// </summary>
        /// <returns></returns>
        public int CountVertices()
        {
            return Xors3d.CountVertices(_handle);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="vertex"></param>
        /// <returns></returns>
        public Vector3 GetVertexPos(int vertex)
        {
            return new Vector3(Xors3d.VertexX(_handle, vertex), Xors3d.VertexY(_handle, vertex), Xors3d.VertexZ(_handle, vertex));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="vertex"></param>
        /// <returns></returns>
        public Vector3 GetVertexNormal(int vertex)
        {
            return new Vector3(Xors3d.VertexNX(_handle, vertex), Xors3d.VertexNY(_handle, vertex), Xors3d.VertexNZ(_handle, vertex));
        }

        public Vector3 GetVertexTangent(int vertex)
        {
            return new Vector3(Xors3d.VertexTX(_handle, vertex), Xors3d.VertexTY(_handle, vertex), Xors3d.VertexTZ(_handle, vertex));
        }

        public Vector3 GetVertexBinormal(int vertex)
        {
            return new Vector3(Xors3d.VertexBX(_handle, vertex), Xors3d.VertexBY(_handle, vertex), Xors3d.VertexBZ(_handle, vertex));
        }

        public Vector3 GetVertexUVW(int vertex)
        {
            return new Vector3(Xors3d.VertexU(_handle, vertex), Xors3d.VertexV(_handle, vertex), Xors3d.VertexW(_handle, vertex));
        }

        public Color GetVertexColor(int vertex)
        {
            return new Color((int)Xors3d.VertexRed(_handle, vertex), (int)Xors3d.VertexGreen(_handle, vertex), (int)Xors3d.VertexBlue(_handle, vertex), (int)Xors3d.VertexAlpha(_handle, vertex));
        }

        /// <summary>
        /// Returns the vertex of a triangle corner
        /// </summary>
        /// <param name="triangle"></param>
        /// <param name="corner">Corner of triangle. Should be 0, 1 or 2.  </param>
        /// <returns></returns>
        public int GetTriangleVertex(int triangle, int corner)
        {
            return Xors3d.TriangleVertex(_handle, triangle, corner);
        }

        /// <summary>
        /// Returns the number of triangles in a surface
        /// </summary>
        /// <returns></returns>
        public int CountTriangles()
        {
            return Xors3d.CountTriangles(_handle);
        }

        /// <summary>
        /// Paints a surface with a brush. 
        /// This has the effect of instantly altering the visible appearance of that particular surface, i.e. section of mesh, assuming the brush's properties are different to what was applied to the surface before
        /// </summary>
        /// <param name="brush"></param>
        public void Paint(Brush brush)
        {
            Xors3d.PaintSurface(_handle, brush._handle);
        }

        /// <summary>
        /// Removes all vertices and/or triangles from a surface. 
        /// This is useful for clearing sections of mesh. The results will be instantly visible. After clearing a surface, you may wish to add vertices and triangles to it again but with a slightly different polygon count for dynamic level of detail (LOD). 
        /// </summary>
        /// <param name="vertices"></param>
        /// <param name="triangles"></param>
        public void ClearSurface(bool vertices = true, bool triangles = true)
        {
            Xors3d.ClearSurface(_handle, vertices, triangles);
        }

        /// <summary>
        /// Return surface texture from specified index
        /// </summary>
        /// <param name="layer"></param>
        /// <returns></returns>
        public Texture GetTexture(int layer=0)
        {
            Texture t = new Texture();
            t._handle = Xors3d.GetSurfaceTexture(_handle, layer);
            return t._handle > 0 ? t : null;
        }

        /// <summary>
        /// Frees up a surface
        /// </summary>
        public void FreeSurface()
        {
            Xors3d.FreeSurface(_handle);
        }

        /// <summary>
        /// Sets the type of primitive to render
        /// </summary>
        /// <param name="ptype"></param>
        public void SetSurfacePrimitiveType(MeshPriminiveType ptype)
        {
            Xors3d.SurfacePrimitiveType(_handle, (int)ptype);
        }

        /// <summary>
        /// Sets the diffuse color of a surface. 
        /// The red, green and blue values should be in the range [0; 255] with 0 being darkest and 255 brightest. The default surface color is 255, 255, 255 (white)
        /// </summary>
        /// <param name="color"></param>
        public void SetSurfaceColor(Color color)
        {
            Xors3d.SurfaceColor(_handle, color._red, color._green, color._blue);
        }

        /// <summary>
        /// Sets the alpha level of a surface. 
        /// The 'alpha' value should be in a floating point value in the range [0.0; 1.0]. The default surface alpha setting is 1.0. The alpha level is how transparent a surface is. A value of 1.0 will mean the surface is opaque. A value of 0.0 will mean the surface is completely transparent, i.e. invisible. Values between 0.0 and 1.0 will cause varying amount of transparency. This is useful for imitating the look of objects such as glass and other translucent materials. 
        /// </summary>
        /// <param name="a"></param>
        public void SetSurfaceAlpha(float a)
        {
            Xors3d.SurfaceAlpha(_handle, a);
        }

        /// <summary>
        /// Sets the specular shininess of a surface. 
        /// The shininess value should be a floting point number in the range [0.0; 1.0]. The default shininess setting is 0.0 Shininess is how much brighter certain areas of an object will appear to be when a light is shone directly at them. Setting a shininess value of 1.0 for a medium to high poly sphere, combined with the creation of a light shining in the direction of it, will give it the appearance of a shiny snooker ball. 
        /// </summary>
        /// <param name="s"></param>
        public void SetSurfaceShininess(float s)
        {
            Xors3d.SurfaceShininess(_handle, s);
        }

        /// <summary>
        /// Sets the blending mode of a surface. 
        /// This blending mode determines the way in which the new RGBA of the pixel being rendered is combined with the RGB of the background. To calculate the new RGBA of the pixel being rendered, the texture RGBA for the pixel is taken, its alpha component multiplied by the surfaces/brushes (where applicable) alpha value and its color compentent multiplied by the surfaces/brushes colour. This is the RGBA which will then be blended into the background pixel, and how this is done depends on the xSurfaceBlend() value. 
        /// </summary>
        /// <param name="blend"></param>
        public void SetSurfaceBlend(Blend blend)
        {
            Xors3d.SurfaceBlend(_handle, (int)blend);
        }

        /// <summary>
        /// Sets miscellaneous effects for a surface. 
        /// Flags can be added to combine two or more effects
        /// </summary>
        /// <param name="fx"></param>
        public void SetSurfaceFX(FX fx)
        {
            Xors3d.SurfaceFX(_handle, (int)fx);
        }

        /// <summary>
        /// Sets the reference alpha value of a surface. 
        /// The 'value' value should be an integer in the range [0; 255]. The default value is 0. It specifies a reference alpha value against which pixels are tested when alpha testing is enabled (See xSurfaceAlphaFunc()). This is useful for varying smoothness of edges of surfaces with masked textures (e.g. grass, leaves, clouds). 
        /// </summary>
        /// <param name="a"></param>
        public void SetSurfaceAlphaRef(int a)
        {
            Xors3d.SurfaceAlphaRef(_handle, a);
        }

        /// <summary>
        /// Sets the alpha test function of a surface. 
        /// The 'value' value should be an integer in the range [1; 8]. The default value is 5 (GREATER). Alpha test enables an application to accept or reject a pixel, based on its alpha value
        /// </summary>
        /// <param name="a"></param>
        public void SetSurfaceAlphaFunc(CompareModes a = CompareModes.GREATER)
        {
            Xors3d.SurfaceAlphaFunc(_handle, (int)a);
        }
    }
}
