﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Texture
    {
        internal int _handle = 0;


        public bool equals(Texture value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Sets texture filtering method
        /// </summary>
        /// <param name="filter"></param>
        public void SetFilter(TextureFilter filter)
        {
            Xors3d.SetTextureFilter(_handle, (int)filter);
        }

        /// <summary>
        /// Returns the handle of a texture's drawing buffer. 
        /// This can be used with xSetBuffer() to perform direct drawing operations to the texture 
        /// </summary>
        /// <param name="frame"></param>
        /// <returns></returns>
        public TextureBuffer GetBuffer(int frame = 0)
        {
            TextureBuffer tb = new TextureBuffer();
            tb._handle = Xors3d.TextureBuffer(_handle, frame);
            return tb._handle > 0 ? tb : null;
        }

        /// <summary>
        /// Copys data from one texture to another using hardware accelerated method. 
        /// </summary>
        /// <param name="x1"></param>
        /// <param name="y1"></param>
        /// <param name="width1"></param>
        /// <param name="height1"></param>
        /// <param name="texture2"></param>
        /// <param name="x2"></param>
        /// <param name="y2"></param>
        /// <param name="width2"></param>
        /// <param name="height2"></param>
        /// <param name="filter"></param>
        public void StretchRect(int x1, int y1, int width1, int height1, Texture texture2, int x2, int y2, int width2, int height2, int filter)
        {
            Xors3d.StretchRect(_handle, x1, y1, width1, height1, texture2._handle, x2, y2, width2, height2, filter);
        }


        public int GetWidth()
        {
            return Xors3d.TextureWidth(_handle);
        }

        public int GetHeight()
        {
            return Xors3d.TextureHeight(_handle);
        }

        /// <summary>
        /// Frees up a texture
        /// </summary>
        public void Free()
        {
            Xors3d.FreeTexture(_handle);
        }

        /// <summary>
        /// Sets the blending mode for a texture. 
        /// The texture blend mode determines how the texture will blend with the texture or polygon which is 'below' it. Texture 0 will blend with the polygons of the entity it is applied to. Texture 1 will blend with texture 0. Texture 2 will blend with texture 1. And so on. Texture blending effectively takes the highest order texture (the one with the highest index) and it blends with the texture below it, then that result to the texture directly below again, and so on until texture 0 which is blended with the polygons of the entity it is applied to and thus the world, depending on the xEntityBlend() of the object. Each of the blend modes are identical to their xEntityBlend() counterparts. 
        /// </summary>
        /// <param name="blend"></param>
        public void SetTextureBlend(TextureBlend blend)
        {
            Xors3d.TextureBlend(_handle, (int)blend);
        }

        /// <summary>
        /// Sets the texture coordinate mode for a texture. 
        /// This determines where the UV values used to look up a texture come from 
        /// </summary>
        /// <param name="coords">UV coordinates number (0 or 1)</param>
        public void SetTextureCoords(int coords)
        {
            Xors3d.TextureCoords(_handle, coords);
        }

        /// <summary>
        /// Returns a texture's absolute filename. 
        /// To find out just the name of the texture, you will need to parse the string returned by xTextureName(). 
        /// </summary>
        public string Name
        {
            get
            {
                return Xors3d.TextureName(_handle);
            }
        }

        /// <summary>
        /// Positions a texture at an absolute position. 
        /// This will have an immediate effect on all instances of the texture being used. Positioning a texture is useful for performing scrolling texture effects, such as for water etc
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void SetPosition(float x, float y)
        {
            Xors3d.PositionTexture(_handle, x, y);
        }

        /// <summary>
        /// Scales a texture by an absolute amount. 
        /// This will have an immediate effect on all instances of the texture being used 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public void SetScale(float x, float y)
        {
            Xors3d.ScaleTexture(_handle, x, y);
        }

        /// <summary>
        /// Rotates a texture. 
        /// This will have an immediate effect on all instances of the texture being used. Rotating a texture is useful for performing swirling texture effects, such as for smoke etc 
        /// </summary>
        /// <param name="angle"></param>
        public void SetRotate(float angle)
        {
            Xors3d.RotateTexture(_handle, angle);
        }

        /// <summary>
        /// Returns texture pixels data. 
        /// NOTE: You may get pixels data only from locked texture frames 
        /// </summary>
        /// <param name="frame"></param>
        /// <returns></returns>
        public IntPtr GetData(int frame=0)
        {
            return new IntPtr( Xors3d.GetTextureData(_handle, frame) );
        }

        /// <summary>
        /// Returns texture pixels data pitch. 
        /// You must use this value for computing pixel position instead texture width (e.g. value = pixels[y * xGetTextureDataPitch() + x]) if you use non power-of-two textures.
        /// NOTE: You can get pixels data only from locked texture frames 
        /// </summary>
        /// <param name="frame"></param>
        /// <returns></returns>
        public int GetDataPitch(int frame=0)
        {
            return Xors3d.GetTextureDataPitch(_handle, frame);
        }

        /// <summary>
        /// Returns a pointer to Direct3D texture surface
        /// </summary>
        /// <param name="frame"></param>
        /// <returns></returns>
        public IntPtr GetSurface(int frame = 0)
        {
            return new IntPtr(Xors3d.GetTextureSurface(_handle, frame));
        }

        /// <summary>
        /// Returns a count of texture frames
        /// </summary>
        /// <returns></returns>
        public int GetFrames()
        {
            return Xors3d.GetTextureFrames(_handle);
        }

        /// <summary>
        /// Selects a cube face for direct rendering to a texture. 
        /// This command should only be used when you wish to draw directly to a cubemap texture in real-time. Otherwise, just loading a pre-rendered cubemap with a flag of FLAGS_CUBICENVMAP will suffice. To understand how this command works exactly it is important to recognise that Xors3D treats cubemap textures slightly differently to how it treats other textures. Here's how it works... A cubemap texture in Xors3D actually consists of six images, each of which must be square 'power' of two size - e.g. 32, 64, 128 etc. Each corresponds to a particular cube face. These images are stored internally by Xors3D, and the texture handle that is returned by xLoadTexture()/xCreateTexture() when specifying the cubemap flag, only provides access to one of these six images at once (by default the first one, or 'FORWARD' face). How do we get access to the other five images? This is where xSetCubeFace() comes in. It will tell Xors3D that whenever you next draw to a cubemap texture, to draw to the particular image representing the face you have specified with the face parameter. 
        /// </summary>
        /// <param name="face"></param>
        public void SetCubeFace(CubeMapFaces face)
        {
            Xors3d.SetCubeFace(_handle, (int)face);
        }

        /// <summary>
        /// Set the rendering mode of a cubemap texture
        /// </summary>
        /// <param name="mode"></param>
        public void SetCubeMode(CubeMapMode mode)
        {
            Xors3d.SetCubeMode(_handle, (int)mode);
        }
    }
}
