﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xors.Physics;

namespace Xors
{
    //MAIN STATIC CLASS OF THE XORS3D
    static public class XE
    {
        static internal List<Behaviour> behavs = new List<Behaviour>();
        static internal List<Entity> bumps = new List<Entity>();

        /// <summary>
        /// Update All Added Behaviours of entities
        /// </summary>
        static public void UpdateBehaviours()
        {
            foreach (Behaviour behav in behavs)
            {
                behav.OnUpdate();
            }
        }


        /// <summary>
        /// Creates a brush, loads and assigns a texture to it, and returns a brush 
        /// </summary>
        /// <param name="path">Filename of texture </param>
        /// <param name="flags">Loading flags. See 'Texture loading flags' for more information </param>
        /// <param name="xScale">Brush u scale </param>
        /// <param name="yScale">Brush v scale </param>
        /// <returns></returns>
        static public Brush LoadBrush(string path, TextureFlags flags=(TextureFlags)9, float xScale=1, float yScale=1)
        {
            Brush b = new Brush();
            b._handle = Xors3d.LoadBrush(path, (int)flags, xScale, yScale);
            return b._handle > 0 ? b : null;
        }

        /// <summary>
        /// Creates a brush and returns a brush handle. 
        /// The green, red and blue values allow you to set the colour of the brush. Values should be in the range [0; 255]. If omitted the values default to 255. A brush is a collection of properties such as color, alpha, shininess, textures, etc that are all stored as part of the brush. Then, all these properties can be applied to an entity, mesh or surface at once just by using xPaintEntity(), xPaintMesh() or xPaintSurface(). When creating your own mesh, if you wish for certain surfaces to look differently from one another, then you will need to use brushes to paint individual surfaces. Using commands such as xEntityColor(), xEntityAlpha() will apply the effect to all surfaces at once, which may not be what you wish to achieve. 
        /// </summary>
        /// <param name="color"></param>
        /// <returns></returns>
        static public Brush CreateBrush(Color color)
        {
            Brush b = new Brush();
            b._handle = Xors3d.CreateBrush(color._red, color._green, color._blue);
            return b._handle > 0 ? b : null;
        }

        /// <summary>
        /// Creates a camera entity and returns its handle. 
        /// Without at least one camera, you won't be able to see anything in your 3D world. With more than one camera, you will be to achieve effect such as split-screen modes and rear-view mirrors. In Xors3D camera can render in any rendering buffer (xBackBuffer(), xImageBuffer(), xTextureBuffer()). The optional parent parameter allow you to specify a parent entity for the camera so that when the parent is moved the child camera will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the camera being created at position 0, 0, 0 rather than at the parent entity's position. 
        /// </summary>
        /// <returns></returns>
        static public Camera CreateCamera(Entity parent=null)
        {
            Camera cam = new Camera();
            cam._handle = Xors3d.CreateCamera(parent != null ? parent._handle : 0);
            if(Shadows.AutoShadows)
                cam.EnableShadows = true;
            return cam._handle > 0 ? cam : null;
        }

        /// <summary>
        /// Returns the world xyz coordinate of the most recently executed pick command
        /// </summary>
        /// <returns></returns>
        static Vector3 GetPickedPos()
        {
            Vector3 v = new Vector3(Xors3d.PickedX(),Xors3d.PickedY(),Xors3d.PickedZ());
            return v;
        }

        /// <summary>
        /// Returns the xyz component of the normal of the most recently executed pick command
        /// </summary>
        /// <returns></returns>
        static Vector3 GetPickedNormals()
        {
            Vector3 v = new Vector3(Xors3d.PickedNX(), Xors3d.PickedNY(), Xors3d.PickedNZ());
            return v;
        }

        /// <summary>
        /// Returns the entity 'picked' by the most recently executed pick command. 
        /// This might have been xCameraPick(), xEntityPick() or xLinePick()
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        static public Entity GetPickedEntity(int x, int y)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.PickedEntity();
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Returns the handle of the surface that was 'picked' by the most recently executed pick command. 
        /// This might have been xCameraPick(), xEntityPick() or xLinePick(). 
        /// </summary>
        /// <returns></returns>
        static public Surface GetPickedSurface()
        {
            Surface s = new Surface();
            s._handle = Xors3d.PickedSurface();
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Returns the index number of the triangle that was 'picked' by the most recently executed pick command. 
        /// This might have been xCameraPick(), xEntityPick() or xLinePick()
        /// </summary>
        /// <returns></returns>
        static public int GetPickedTriangle()
        {
            return Xors3d.PickedTriangle();
        }

        /// <summary>
        /// Returns the time taken to calculate the most recently executed pick command. 
        /// This might have been xCameraPick(), xEntityPick() or xLinePick(). 
        /// </summary>
        /// <returns></returns>
        static public int PickedTime()
        {
            return Xors3d.PickedTime();
        }

        static public void SetCollisions(int srcType, int destType, CollisionsMode mode, CollisionsResponce responce)
        {
            Xors3d.Collisions(srcType, destType, (int)mode, (int)responce);
        }

        static public void ClearCollisions()
        {
            Xors3d.ClearCollisions();
        }

        /// <summary>
        /// Load animated mesh
        /// </summary>
        /// <param name="path"></param>
        /// <param name="parent"></param>
        /// <returns></returns>
        static public AnimatedMesh LoadAnimMesh(string path, Entity parent)
        {
            AnimatedMesh m = new AnimatedMesh();
            m._handle = Xors3d.LoadAnimMesh(path, parent != null ? parent._handle : 0);
            return m._handle > 0 ? m : null;
        }

        /// <summary>
        /// Activates a full Xors3D version
        /// </summary>
        /// <param name="key"></param>
        static public void Key(string key)
        {
            Xors3d.Key(key);
        }

        /// <summary>
        /// Sets the value for engine's parameter
        /// </summary>
        /// <param name="param"></param>
        /// <param name="value"></param>
        static public void SetEngineSettings(string param, string value)
        {
            Xors3d.SetEngineSetting(param, value);
        }

        /// <summary>
        /// Returns the value of engine's parameter
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        static public string GetEngineSettings(string param)
        {
            return Xors3d.GetEngineSetting(param);
        }

        /// <summary>
        /// Creates a new image with a single frame or multiple frames for animation
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="frame">Number of frames </param>
        /// <returns></returns>
        static public Sprite CreateImage(int width, int height, int frame = 1)
        {
            Sprite s = new Sprite();
            s._handle = Xors3d.CreateImage(width, height, frame);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// This command loads an image. 
        /// You will use the xDrawImage() command to display the graphic later
        /// </summary>
        /// <param name="path">String designating full path and filename to image </param>
        /// <returns></returns>
        static public Sprite LoadImage(string path)
        {
            Sprite s = new Sprite();
            s._handle = Xors3d.LoadImage(path);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Load's and animated image. 
        /// While similar to xLoadImage(), the xLoadAnimImage() loads a single image that is made up of 'frames' of seperate images (presumably to be used as frames of a graphic animation). The imagestrip itself consists of 2 or more frames, arranged in a single graphic image. There is no spaces between the frames, and each frame must be the same width and height. When loaded, the frames will be indexed in a left-to-right, top-to-bottom fashion, starting in the top left corner. When drawing the image to the screen with the xDrawImage() command, you specify which frame to draw with the frame parameter. To actually make your image animate, you'll need to cycle through the frames (like a flip book, cartoon, or any other video) quickly to give the illusion of motion. Our example will show you how to use one of the sample imagestrips and make it animate. While it may seem confusing, we are going to do some timer work as well as a little weird math. 
        /// </summary>
        /// <param name="path">String designating full path and filename to image </param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="startFrame"></param>
        /// <param name="frames"></param>
        /// <returns></returns>
        static public Sprite LoadAnimImage(string path, int width, int height, int startFrame, int frames)
        {
            Sprite s = new Sprite();
            s._handle = Xors3d.LoadAnimImage(path, width, height, startFrame, frames);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Creates a light. 
        /// Lights work by affecting the colour of all vertices within the light's range. You should create at least one light if you wish to use 3D graphics otherwise everything will appear flat
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        static public Light CreateLight(LightType type)
        {
            Light l = new Light();
            l._handle = Xors3d.CreateLight((int)type);
            if (Shadows.AutoShadows)
            {
                l.EnableShadows = true;
                l.SetShadowEpsilons(0.0001f, 0.16f);
            }
            return l._handle > 0 ? l : null;
        }

        /// <summary>
        /// Creates a sprite entity and returns its handle. Sprites are simple flat (usually textured) rectangles made from two triangles. The sprite will be positioned at 0, 0, 0 and extend from 1, -1 to +1, +1. Sprites have two real strengths. The first is that they consist of only two polygons; meaning you can use many of them at once. This makes them ideal for particle effects and 2D-using-3D games where you want lots of sprites on-screen at once. Secondly, sprites can be assigned a view mode using cSpriteViewMode(). By default this view mode is set to FIXED, which means the sprite will always face the camera. So no matter what the orientation of the camera is relative to the sprite, you will never actually notice that they are flat; by giving them a spherical texture, you can make them appear to look no different than a normal sphere. The parent parameter allow you to specify a parent entity for the sprite so that when the parent is moved the child sprite will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the sprite being created at position 0, 0, 0 rather than at the parent entity's position. Note: Sprites have their own commands for rotation and scaling. 
        /// </summary>
        /// <returns></returns>
        static public SpriteEntity CreateSprite(Entity parent=null)
        {
            SpriteEntity s = new SpriteEntity();
            s._handle = Xors3d.CreateSprite(parent != null ? parent._handle : 0);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Creates a sprite entity, assigns a texture to it, and retuns sprite handle
        /// </summary>
        /// <param name="path"></param>
        /// <param name="flags"></param>
        /// <param name="parent"></param>
        /// <returns></returns>
        static public SpriteEntity LoadSprite(string path, TextureFlags flags = (TextureFlags)9, Entity parent = null)
        {
            SpriteEntity s = new SpriteEntity();
            s._handle = Xors3d.LoadSprite(path, (int)flags, parent != null ? parent._handle : 0);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Creates a terrain entity and returns its handle. 
        /// The terrain extends from 0, 0, 0 to size, 1, size. The parent parameter allows you to specify a parent entity for the terrain so that when the parent is moved the child terrain will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the terrain being created at position 0, 0, 0 rather than at the parent entity's position. 
        /// </summary>
        /// <param name="size">Number of grid squares along each side of terrain, and must be a power of 2 value, e.g. 32, 64, 128, 256, 512, 1024</param>
        /// <param name="parent"></param>
        /// <returns></returns>
        static public Terrain CreateTerrain(int size , Entity parent)
        {
            Terrain t = new Terrain();
            t._handle = Xors3d.CreateTerrain(size, parent != null ? parent._handle : 0);
            return t._handle > 0 ? t : null;
        }

        /// <summary>
        /// Loads a terrain from an image file and returns the terrain's handle. 
        /// The image's red channel is used to determine heights. Terrain is initially the same width and depth as the image, and 1.0 unit high.
        /// Tips on generating nice terrain:
        /// Smooth or blur the height map
        /// Reduce the y scale of the terrain
        /// Increase the x/z scale of the terrain
        /// Reduce the camera range
        /// When texturing an entity, a texture with a scale of 1, 1, 1 (default) will cover the whole terrain.
        ///The parent parameter allows you to specify a parent entity for the terrain so that when the parent is moved the child terrain will move with it. However, this relationship is one way; applying movement commands to the child will not affect the parent. Specifying a parent entity will still result in the terrain being created at position 0, 0, 0 rather than at the parent entity's position. A heightmaps dimensions (width and height) must be the same and must be a power of 2 (e.g. 64, 128, 256, 512, 1024) or a power of 2 + 1 (e.g. 65, 129, 257, 513, 1025). 
        /// </summary>
        /// <param name="path"></param>
        /// <param name="parent"></param>
        /// <returns></returns>
        static public Terrain LoadTerrain(string path, Entity parent=null)
        {
            Terrain t = new Terrain();
            t._handle = Xors3d.LoadTerrain(path, parent != null ? parent._handle : 0);
            return t._handle > 0 ? t : null;
        }

        /// <summary>
        /// Loads the streaming texture for terrains
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        static public TerrainTexture LoadTerrainTexture(string path)
        {
            TerrainTexture t = new TerrainTexture();
            t._handle = Xors3d.LoadTerrainTexture(path);
            return t._handle > 0 ? t : null;
        }

        /// <summary>
        /// Creates a texture and returns its handle. 
        /// Width and height are the size of the texture. Note that the actual texture size may be different from the width and height requested, as different types of 3D hardware support different sizes of texture. The optional flags parameter allows you to apply certain effects to the texture. Flags can be added to combine two or more effects. See 'Texture loading flags' for more information Once you have created a texture, use xSetBuffer(xTextureBuffer()) to draw to it. Xors3D supports direct rendering into textures. 
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="flags"></param>
        /// <param name="frames"></param>
        /// <returns></returns>
        static public Texture CreateTexture(int width, int height, TextureFlags flags = (TextureFlags)9, int frames = 1)
        {
            Texture t = new Texture();
            t._handle = Xors3d.CreateTexture(width, height, (int)flags, frames);
            return t._handle > 0 ? t : null;
        }

        /// <summary>
        /// Sets texture filtering. 
        /// Loaded textures that contain text specified by 'matchText' in file name param will have the provided flags added. 
        /// </summary>
        /// <param name="matchText">Text string that, if found in texture filename, will activate certain filters</param>
        /// <param name="filter"></param>
        static public void SetTextureFilter(string matchText, TextureFilter filter)
        {
            Xors3d.TextureFilter(matchText, (int)filter);
        }

        /// <summary>
        /// Loads a texture from an image file and returns the texture's handle. 
        /// The optional flags parameter allows you to apply certain effects to the texture. Flags can be added to combine two or more effects. See 'Texture loading flags' for more infromation. Supported file formats are: bmp, dds, dib, hdr, jpg, pfm, png, ppm, tga
        /// </summary>
        /// <param name="path"></param>
        /// <param name="flags"></param>
        /// <returns></returns>
        static public Texture LoadTexture(string path, TextureFlags flags = (TextureFlags)9)
        {
            Texture t = new Texture();
            t._handle = Xors3d.LoadTexture(path, (int)flags);
            return t._handle > 0 ? t : null;
        }

        /// <summary>
        /// Loads an animated texture. 
        /// The 'flags' parameter allows you to apply certain effects to the texture. Flags can be added to combine two or more effects. See 'Texture loading flags' for more infromation. 
        /// </summary>
        /// <param name="path"></param>
        /// <param name="flags"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="startFrame"></param>
        /// <param name="frames"></param>
        /// <returns></returns>
        static public Texture LoadAnimTexture(string path, TextureFlags flags, int width, int height, int startFrame, int frames)
        {
            Texture t = new Texture();
            t._handle = Xors3d.LoadAnimTexture(path, (int)flags,width,height,startFrame,frames);
            return t._handle > 0 ? t : null;
        }

        /// <summary>
        /// Creates texture from user data. 
        /// </summary>
        /// <param name="pixelsData"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="flags"></param>
        /// <param name="frames"></param>
        /// <returns></returns>
        static public Texture CreateTextureFromData(IntPtr pixelsData, int width, int height, TextureFlags flags = (TextureFlags)9, int frames = 1)
        {
            Texture t = new Texture();
            t._handle = Xors3d.CreateTextureFromData(pixelsData.ToInt32(),width,height,(int)flags,frames);
            return t._handle > 0 ? t : null;
        }


        static public Entity CreatePivot(Entity parent = null)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.CreatePivot(parent != null ? parent._handle : 0);
            return ent._handle > 0 ? ent : null;
        }
        
    }
}
